# -*- coding:utf-8 -*-

from .models.model_serverplan import Model_ServerPlan
from .models.model_diskplan import Model_DiskPlan
from .models.model_routerplan import Model_RouterPlan
from .models.model_licenseinfo import Model_LicenseInfo
from .client import Client
from ..util import Util

# module saklient.cloud.product

class Product:
    ## 商品情報にアクセスするためのモデルを集めたクラス。
    
    # (instance field) _server
    
    ## @return {saklient.cloud.models.model_serverplan.Model_ServerPlan}
    def get_server(self):
        return self._server
    
    ## サーバプラン情報。
    server = property(get_server, None, None)
    
    # (instance field) _disk
    
    ## @return {saklient.cloud.models.model_diskplan.Model_DiskPlan}
    def get_disk(self):
        return self._disk
    
    ## ディスクプラン情報。
    disk = property(get_disk, None, None)
    
    # (instance field) _router
    
    ## @return {saklient.cloud.models.model_routerplan.Model_RouterPlan}
    def get_router(self):
        return self._router
    
    ## ルータ帯域プラン情報。
    router = property(get_router, None, None)
    
    # (instance field) _license
    
    ## @return {saklient.cloud.models.model_licenseinfo.Model_LicenseInfo}
    def get_license(self):
        return self._license
    
    ## ライセンス種別情報。
    license = property(get_license, None, None)
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    def __init__(self, client):
        Util.validate_type(client, "saklient.cloud.client.Client")
        self._server = Model_ServerPlan(client)
        self._disk = Model_DiskPlan(client)
        self._router = Model_RouterPlan(client)
        self._license = Model_LicenseInfo(client)
    



# -*- coding:utf-8 -*-

from ...errors.httpnotfoundexception import HttpNotFoundException

# module saklient.cloud.errors.invaliduriargumentexception

class InvalidUriArgumentException(HttpNotFoundException):
    ## 対象が見つかりません。パスに使用できない文字が含まれています。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(InvalidUriArgumentException, self).__init__(status, code, "対象が見つかりません。パスに使用できない文字が含まれています。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.httpconflictexception import HttpConflictException

# module saklient.cloud.errors.packetfilterversionmismatchexception

class PacketFilterVersionMismatchException(HttpConflictException):
    ## 要求された操作を行えません。起動中のサーバが収容されているハイパーバイザとパケットフィルタのバージョンが合致しません。コントロールパネルまたはAPIからの操作によりサーバを一旦停止し、再度起動後にお試しください。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(PacketFilterVersionMismatchException, self).__init__(status, code, "要求された操作を行えません。起動中のサーバが収容されているハイパーバイザとパケットフィルタのバージョンが合致しません。コントロールパネルまたはAPIからの操作によりサーバを一旦停止し、再度起動後にお試しください。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ..client import Client
from .resource import Resource
from ...util import Util

# module saklient.cloud.resources.ipv4net

class Ipv4Net(Resource):
    ## IPv4ネットワークの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_address
    
    # (instance field) m_mask_len
    
    # (instance field) m_default_route
    
    # (instance field) m_next_hop
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/subnet"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Subnet"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Subnets"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Ipv4Net"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {Swytch} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(Ipv4Net, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_address = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_address(self):
        return self.m_address
    
    ## ネットワークアドレス
    address = property(get_address, None, None)
    
    # (instance field) n_mask_len = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_mask_len(self):
        return self.m_mask_len
    
    ## マスク長
    mask_len = property(get_mask_len, None, None)
    
    # (instance field) n_default_route = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_default_route(self):
        return self.m_default_route
    
    ## デフォルトルート
    default_route = property(get_default_route, None, None)
    
    # (instance field) n_next_hop = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_next_hop(self):
        return self.m_next_hop
    
    ## ネクストホップ
    next_hop = property(get_next_hop, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "NetworkAddress"):
            self.m_address = None if Util.get_by_path(r, "NetworkAddress") is None else str(Util.get_by_path(r, "NetworkAddress"))
        else:
            self.m_address = None
            self.is_incomplete = True
        self.n_address = False
        if Util.exists_path(r, "NetworkMaskLen"):
            self.m_mask_len = None if Util.get_by_path(r, "NetworkMaskLen") is None else int(str(Util.get_by_path(r, "NetworkMaskLen")))
        else:
            self.m_mask_len = None
            self.is_incomplete = True
        self.n_mask_len = False
        if Util.exists_path(r, "DefaultRoute"):
            self.m_default_route = None if Util.get_by_path(r, "DefaultRoute") is None else str(Util.get_by_path(r, "DefaultRoute"))
        else:
            self.m_default_route = None
            self.is_incomplete = True
        self.n_default_route = False
        if Util.exists_path(r, "NextHop"):
            self.m_next_hop = None if Util.get_by_path(r, "NextHop") is None else str(Util.get_by_path(r, "NextHop"))
        else:
            self.m_next_hop = None
            self.is_incomplete = True
        self.n_next_hop = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_address:
            Util.set_by_path(ret, "NetworkAddress", self.m_address)
        if withClean or self.n_mask_len:
            Util.set_by_path(ret, "NetworkMaskLen", self.m_mask_len)
        if withClean or self.n_default_route:
            Util.set_by_path(ret, "DefaultRoute", self.m_default_route)
        if withClean or self.n_next_hop:
            Util.set_by_path(ret, "NextHop", self.m_next_hop)
        return ret
    



# -*- coding:utf-8 -*-

from ...errors.saklientexception import SaklientException
from ..client import Client
from .resource import Resource
from .swytch import Swytch
from ...util import Util

# module saklient.cloud.resources.iface

class Iface(Resource):
    ## インタフェースの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_mac_address
    
    # (instance field) m_ip_address
    
    # (instance field) m_user_ip_address
    
    # (instance field) m_server_id
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/interface"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Interface"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Interfaces"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Iface"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## このローカルオブジェクトに現在設定されているリソース情報をAPIに送信し、新規作成または上書き保存します。
    # 
    # @return {saklient.cloud.resources.iface.Iface} this
    def save(self):
        return self._save()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {saklient.cloud.resources.iface.Iface} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(Iface, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    ## スイッチに接続します。
    # 
    # @param {saklient.cloud.resources.swytch.Swytch} swytch 接続先のスイッチ。
    # @return {saklient.cloud.resources.iface.Iface} this
    def connect_to_swytch(self, swytch):
        Util.validate_type(swytch, "saklient.cloud.resources.swytch.Swytch")
        self._client.request("PUT", self._api_path() + "/" + Util.url_encode(self._id()) + "/to/switch/" + Util.url_encode(swytch._id()))
        return self.reload()
    
    ## 共有セグメントに接続します。
    # 
    # @return {saklient.cloud.resources.iface.Iface} this
    def connect_to_shared_segment(self):
        self._client.request("PUT", self._api_path() + "/" + Util.url_encode(self._id()) + "/to/switch/shared")
        return self.reload()
    
    ## スイッチから切断します。
    # 
    # @return {saklient.cloud.resources.iface.Iface} this
    def disconnect_from_swytch(self):
        self._client.request("DELETE", self._api_path() + "/" + Util.url_encode(self._id()) + "/to/switch")
        return self.reload()
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_mac_address = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_mac_address(self):
        return self.m_mac_address
    
    ## MACアドレス
    mac_address = property(get_mac_address, None, None)
    
    # (instance field) n_ip_address = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_ip_address(self):
        return self.m_ip_address
    
    ## IPv4アドレス（共有セグメントによる自動割当）
    ip_address = property(get_ip_address, None, None)
    
    # (instance field) n_user_ip_address = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_user_ip_address(self):
        return self.m_user_ip_address
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_user_ip_address(self, v):
        Util.validate_type(v, "str")
        self.m_user_ip_address = v
        self.n_user_ip_address = True
        return self.m_user_ip_address
    
    ## ユーザ設定IPv4アドレス
    user_ip_address = property(get_user_ip_address, set_user_ip_address, None)
    
    # (instance field) n_server_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_server_id(self):
        return self.m_server_id
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_server_id(self, v):
        Util.validate_type(v, "str")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.iface.Iface#server_id")
        self.m_server_id = v
        self.n_server_id = True
        return self.m_server_id
    
    ## このインタフェースが取り付けられているサーバのID
    server_id = property(get_server_id, set_server_id, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "MACAddress"):
            self.m_mac_address = None if Util.get_by_path(r, "MACAddress") is None else str(Util.get_by_path(r, "MACAddress"))
        else:
            self.m_mac_address = None
            self.is_incomplete = True
        self.n_mac_address = False
        if Util.exists_path(r, "IPAddress"):
            self.m_ip_address = None if Util.get_by_path(r, "IPAddress") is None else str(Util.get_by_path(r, "IPAddress"))
        else:
            self.m_ip_address = None
            self.is_incomplete = True
        self.n_ip_address = False
        if Util.exists_path(r, "UserIPAddress"):
            self.m_user_ip_address = None if Util.get_by_path(r, "UserIPAddress") is None else str(Util.get_by_path(r, "UserIPAddress"))
        else:
            self.m_user_ip_address = None
            self.is_incomplete = True
        self.n_user_ip_address = False
        if Util.exists_path(r, "Server.ID"):
            self.m_server_id = None if Util.get_by_path(r, "Server.ID") is None else str(Util.get_by_path(r, "Server.ID"))
        else:
            self.m_server_id = None
            self.is_incomplete = True
        self.n_server_id = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        missing = []
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_mac_address:
            Util.set_by_path(ret, "MACAddress", self.m_mac_address)
        if withClean or self.n_ip_address:
            Util.set_by_path(ret, "IPAddress", self.m_ip_address)
        if withClean or self.n_user_ip_address:
            Util.set_by_path(ret, "UserIPAddress", self.m_user_ip_address)
        if withClean or self.n_server_id:
            Util.set_by_path(ret, "Server.ID", self.m_server_id)
        else:
            if self.is_new:
                missing.append("server_id")
        if len(missing) > 0:
            raise SaklientException("required_field", "Required fields must be set before the Iface creation: " + ", ".join(missing))
        return ret
    



# -*- coding:utf-8 -*-

from ...errors.saklientexception import SaklientException
from ..client import Client
from .resource import Resource
from .icon import Icon
from .iface import Iface
from .serverplan import ServerPlan
from .serverinstance import ServerInstance
from .isoimage import IsoImage
from ..enums.eserverinstancestatus import EServerInstanceStatus
from ..enums.eavailability import EAvailability
from ...util import Util

# module saklient.cloud.resources.server

class Server(Resource):
    ## サーバの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_name
    
    # (instance field) m_description
    
    # (instance field) m_tags
    
    # (instance field) m_icon
    
    # (instance field) m_plan
    
    # (instance field) m_ifaces
    
    # (instance field) m_instance
    
    # (instance field) m_availability
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/server"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Server"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Servers"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Server"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## このローカルオブジェクトに現在設定されているリソース情報をAPIに送信し、新規作成または上書き保存します。
    # 
    # @return {saklient.cloud.resources.server.Server} this
    def save(self):
        return self._save()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {saklient.cloud.resources.server.Server} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(Server, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    ## サーバが起動しているときtrueを返します。
    # 
    # @return {bool}
    def is_up(self):
        return self.get_instance().is_up()
    
    ## サーバが停止しているときtrueを返します。
    # 
    # @return {bool}
    def is_down(self):
        return self.get_instance().is_down()
    
    ## サーバを起動します。
    # 
    # @return {saklient.cloud.resources.server.Server} this
    def boot(self):
        self._client.request("PUT", self._api_path() + "/" + Util.url_encode(self._id()) + "/power")
        return self.reload()
    
    ## サーバをシャットダウンします。
    # 
    # @return {saklient.cloud.resources.server.Server} this
    def shutdown(self):
        self._client.request("DELETE", self._api_path() + "/" + Util.url_encode(self._id()) + "/power")
        return self.reload()
    
    ## サーバを強制停止します。
    # 
    # @return {saklient.cloud.resources.server.Server} this
    def stop(self):
        self._client.request("DELETE", self._api_path() + "/" + Util.url_encode(self._id()) + "/power", {
            'Force': True
        })
        return self.reload()
    
    ## サーバを強制再起動します。
    # 
    # @return {saklient.cloud.resources.server.Server} this
    def reboot(self):
        self._client.request("PUT", self._api_path() + "/" + Util.url_encode(self._id()) + "/reset")
        return self.reload()
    
    ## サーバが起動するまで待機します。
    # 
    # @param {int} timeoutSec=180
    # @return {bool}
    def sleep_until_up(self, timeoutSec=180):
        Util.validate_type(timeoutSec, "int")
        return self.sleep_until(EServerInstanceStatus.up, timeoutSec)
    
    ## サーバが停止するまで待機します。
    # 
    # @param {int} timeoutSec=180
    # @return {bool} 成功時はtrue、タイムアウトやエラーによる失敗時はfalseを返します。
    def sleep_until_down(self, timeoutSec=180):
        Util.validate_type(timeoutSec, "int")
        return self.sleep_until(EServerInstanceStatus.down, timeoutSec)
    
    ## サーバが指定のステータスに遷移するまで待機します。
    # 
    # @ignore
    # @param {str} status
    # @param {int} timeoutSec=180
    # @return {bool}
    def sleep_until(self, status, timeoutSec=180):
        Util.validate_type(status, "str")
        Util.validate_type(timeoutSec, "int")
        step = 10
        while (0 < timeoutSec):
            self.reload()
            s = None
            inst = self.instance
            if inst is not None:
                s = inst.status
            if s is None:
                s = EServerInstanceStatus.down
            if s == status:
                return True
            timeoutSec -= step
            if 0 < timeoutSec:
                Util.sleep(step)
        return False
    
    ## サーバプランを変更します。
    # 
    # 成功時はリソースIDが変わることにご注意ください。
    # 
    # @param {saklient.cloud.resources.serverplan.ServerPlan} planTo
    # @return {saklient.cloud.resources.server.Server} this
    def change_plan(self, planTo):
        Util.validate_type(planTo, "saklient.cloud.resources.serverplan.ServerPlan")
        path = self._api_path() + "/" + Util.url_encode(self._id()) + "/to/plan/" + Util.url_encode(planTo._id())
        result = self._client.request("PUT", path)
        self.api_deserialize(result, True)
        return self
    
    ## サーバにインタフェースを1つ増設し、それを取得します。
    # 
    # @return {saklient.cloud.resources.iface.Iface} 増設されたインタフェース
    def add_iface(self):
        model = Util.create_class_instance("saklient.cloud.models.Model_Iface", [self._client])
        res = model.create()
        res.server_id = self._id()
        return res.save()
    
    ## サーバにISOイメージを挿入します。
    # 
    # @param {saklient.cloud.resources.isoimage.IsoImage} iso
    # @return {saklient.cloud.resources.server.Server} this
    def insert_iso_image(self, iso):
        Util.validate_type(iso, "saklient.cloud.resources.isoimage.IsoImage")
        path = self._api_path() + "/" + Util.url_encode(self._id()) + "/cdrom"
        q = {
            'CDROM': {
                'ID': iso._id()
            }
        }
        self._client.request("PUT", path, q)
        self.reload()
        return self
    
    ## サーバに挿入されているISOイメージを排出します。
    # 
    # @return {saklient.cloud.resources.server.Server} this
    def eject_iso_image(self):
        path = self._api_path() + "/" + Util.url_encode(self._id()) + "/cdrom"
        self._client.request("DELETE", path)
        self.reload()
        return self
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_name = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_name(self):
        return self.m_name
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_name(self, v):
        Util.validate_type(v, "str")
        self.m_name = v
        self.n_name = True
        return self.m_name
    
    ## 名前
    name = property(get_name, set_name, None)
    
    # (instance field) n_description = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_description(self):
        return self.m_description
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_description(self, v):
        Util.validate_type(v, "str")
        self.m_description = v
        self.n_description = True
        return self.m_description
    
    ## 説明
    description = property(get_description, set_description, None)
    
    # (instance field) n_tags = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str[]}
    def get_tags(self):
        self.n_tags = True
        return self.m_tags
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str[]} v
    # @return {str[]}
    def set_tags(self, v):
        Util.validate_type(v, "list")
        self.m_tags = v
        self.n_tags = True
        return self.m_tags
    
    ## タグ文字列の配列
    tags = property(get_tags, set_tags, None)
    
    # (instance field) n_icon = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.icon.Icon}
    def get_icon(self):
        return self.m_icon
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {saklient.cloud.resources.icon.Icon} v
    # @return {saklient.cloud.resources.icon.Icon}
    def set_icon(self, v):
        Util.validate_type(v, "saklient.cloud.resources.icon.Icon")
        self.m_icon = v
        self.n_icon = True
        return self.m_icon
    
    ## アイコン
    icon = property(get_icon, set_icon, None)
    
    # (instance field) n_plan = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.serverplan.ServerPlan}
    def get_plan(self):
        return self.m_plan
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {saklient.cloud.resources.serverplan.ServerPlan} v
    # @return {saklient.cloud.resources.serverplan.ServerPlan}
    def set_plan(self, v):
        Util.validate_type(v, "saklient.cloud.resources.serverplan.ServerPlan")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.server.Server#plan")
        self.m_plan = v
        self.n_plan = True
        return self.m_plan
    
    ## プラン
    plan = property(get_plan, set_plan, None)
    
    # (instance field) n_ifaces = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.iface.Iface[]}
    def get_ifaces(self):
        return self.m_ifaces
    
    ## インタフェース
    ifaces = property(get_ifaces, None, None)
    
    # (instance field) n_instance = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.serverinstance.ServerInstance}
    def get_instance(self):
        return self.m_instance
    
    ## インスタンス情報
    instance = property(get_instance, None, None)
    
    # (instance field) n_availability = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_availability(self):
        return self.m_availability
    
    ## 有効状態 {@link EAvailability}
    availability = property(get_availability, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "Name"):
            self.m_name = None if Util.get_by_path(r, "Name") is None else str(Util.get_by_path(r, "Name"))
        else:
            self.m_name = None
            self.is_incomplete = True
        self.n_name = False
        if Util.exists_path(r, "Description"):
            self.m_description = None if Util.get_by_path(r, "Description") is None else str(Util.get_by_path(r, "Description"))
        else:
            self.m_description = None
            self.is_incomplete = True
        self.n_description = False
        if Util.exists_path(r, "Tags"):
            if Util.get_by_path(r, "Tags") is None:
                self.m_tags = []
            else:
                self.m_tags = []
                for t in Util.get_by_path(r, "Tags"):
                    v1 = None
                    v1 = None if t is None else str(t)
                    self.m_tags.append(v1)
        else:
            self.m_tags = None
            self.is_incomplete = True
        self.n_tags = False
        if Util.exists_path(r, "Icon"):
            self.m_icon = None if Util.get_by_path(r, "Icon") is None else Icon(self._client, Util.get_by_path(r, "Icon"))
        else:
            self.m_icon = None
            self.is_incomplete = True
        self.n_icon = False
        if Util.exists_path(r, "ServerPlan"):
            self.m_plan = None if Util.get_by_path(r, "ServerPlan") is None else ServerPlan(self._client, Util.get_by_path(r, "ServerPlan"))
        else:
            self.m_plan = None
            self.is_incomplete = True
        self.n_plan = False
        if Util.exists_path(r, "Interfaces"):
            if Util.get_by_path(r, "Interfaces") is None:
                self.m_ifaces = []
            else:
                self.m_ifaces = []
                for t in Util.get_by_path(r, "Interfaces"):
                    v2 = None
                    v2 = None if t is None else Iface(self._client, t)
                    self.m_ifaces.append(v2)
        else:
            self.m_ifaces = None
            self.is_incomplete = True
        self.n_ifaces = False
        if Util.exists_path(r, "Instance"):
            self.m_instance = None if Util.get_by_path(r, "Instance") is None else ServerInstance(self._client, Util.get_by_path(r, "Instance"))
        else:
            self.m_instance = None
            self.is_incomplete = True
        self.n_instance = False
        if Util.exists_path(r, "Availability"):
            self.m_availability = None if Util.get_by_path(r, "Availability") is None else str(Util.get_by_path(r, "Availability"))
        else:
            self.m_availability = None
            self.is_incomplete = True
        self.n_availability = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        missing = []
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_name:
            Util.set_by_path(ret, "Name", self.m_name)
        else:
            if self.is_new:
                missing.append("name")
        if withClean or self.n_description:
            Util.set_by_path(ret, "Description", self.m_description)
        if withClean or self.n_tags:
            Util.set_by_path(ret, "Tags", [])
            for r1 in self.m_tags:
                v = None
                v = r1
                (ret["Tags"] if "Tags" in ret else None).append(v)
        if withClean or self.n_icon:
            Util.set_by_path(ret, "Icon", (None if self.m_icon is None else self.m_icon.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_icon is None else self.m_icon.api_serialize_id()))
        if withClean or self.n_plan:
            Util.set_by_path(ret, "ServerPlan", (None if self.m_plan is None else self.m_plan.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_plan is None else self.m_plan.api_serialize_id()))
        else:
            if self.is_new:
                missing.append("plan")
        if withClean or self.n_ifaces:
            Util.set_by_path(ret, "Interfaces", [])
            for r2 in self.m_ifaces:
                v = None
                v = (None if r2 is None else r2.api_serialize(withClean)) if withClean else ({
                    'ID': "0"
                } if r2 is None else r2.api_serialize_id())
                (ret["Interfaces"] if "Interfaces" in ret else None).append(v)
        if withClean or self.n_instance:
            Util.set_by_path(ret, "Instance", (None if self.m_instance is None else self.m_instance.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_instance is None else self.m_instance.api_serialize_id()))
        if withClean or self.n_availability:
            Util.set_by_path(ret, "Availability", self.m_availability)
        if len(missing) > 0:
            raise SaklientException("required_field", "Required fields must be set before the Server creation: " + ", ".join(missing))
        return ret
    



# -*- coding:utf-8 -*-

from ...util import Util

# module saklient.cloud.enums.eavailability

class EAvailability:
    ## リソースの有効性を表す列挙子。
    
    SELECTABLE = "selectable"
    selectable = "selectable"
    
    MIGRATING = "migrating"
    migrating = "migrating"
    
    PRECREATE = "precreate"
    precreate = "precreate"
    
    REPLICATING = "replicating"
    replicating = "replicating"
    
    TRANSFERING = "transfering"
    transfering = "transfering"
    
    STOPPED = "stopped"
    stopped = "stopped"
    
    FAILED = "failed"
    failed = "failed"
    
    CHARGED = "charged"
    charged = "charged"
    
    UPLOADING = "uploading"
    uploading = "uploading"
    
    AVAILABLE = "available"
    available = "available"
    
    ## @ignore
    _MAP = {
        "SELECTABLE": 69,
        "MIGRATING": 70,
        "PRECREATE": 71,
        "REPLICATING": 72,
        "TRANSFERING": 73,
        "STOPPED": 75,
        "FAILED": 78,
        "CHARGED": 79,
        "UPLOADING": 80,
        "AVAILABLE": 100
    }
    
    ## @ignore
    @classmethod
    def compare(clazz, lhs, rhs):
        if not (isinstance(lhs, str) and isinstance(rhs, str)): return None
        lhs = lhs.upper()
        rhs = rhs.upper()
        if lhs not in clazz._MAP or rhs not in clazz._MAP: return None
        d = clazz._MAP[lhs] - clazz._MAP[rhs]
        return (0<d) - (d<0)
    
    



# -*- coding:utf-8 -*-

from .httpexception import HttpException

# module saklient.errors.httpnotfoundexception

class HttpNotFoundException(HttpException):
    ## 対象が見つかりません。対象は利用できない状態か、IDまたはパスに誤りがあります。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(HttpNotFoundException, self).__init__(status, code, "対象が見つかりません。対象は利用できない状態か、IDまたはパスに誤りがあります。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...util import Util

# module saklient.cloud.resources.lbserver

class LbServer:
    ## ロードバランサの監視対象サーバ。
    
    # (instance field) _ip_address
    
    ## @return {str}
    def get_ip_address(self):
        return self._ip_address
    
    ## @param {str} v
    # @return {str}
    def set_ip_address(self, v):
        Util.validate_type(v, "str")
        self._ip_address = v
        return self._ip_address
    
    ## IPアドレス
    ip_address = property(get_ip_address, set_ip_address, None)
    
    # (instance field) _port
    
    ## @return {int}
    def get_port(self):
        return self._port
    
    ## @param {int} v
    # @return {int}
    def set_port(self, v):
        Util.validate_type(v, "int")
        self._port = v
        return self._port
    
    ## ポート番号
    port = property(get_port, set_port, None)
    
    # (instance field) _protocol
    
    ## @return {str}
    def get_protocol(self):
        return self._protocol
    
    ## @param {str} v
    # @return {str}
    def set_protocol(self, v):
        Util.validate_type(v, "str")
        self._protocol = v
        return self._protocol
    
    ## 監視方法
    protocol = property(get_protocol, set_protocol, None)
    
    # (instance field) _path_to_check
    
    ## @return {str}
    def get_path_to_check(self):
        return self._path_to_check
    
    ## @param {str} v
    # @return {str}
    def set_path_to_check(self, v):
        Util.validate_type(v, "str")
        self._path_to_check = v
        return self._path_to_check
    
    ## 監視対象パス
    path_to_check = property(get_path_to_check, set_path_to_check, None)
    
    # (instance field) _response_expected
    
    ## @return {int}
    def get_response_expected(self):
        return self._response_expected
    
    ## @param {int} v
    # @return {int}
    def set_response_expected(self, v):
        Util.validate_type(v, "int")
        self._response_expected = v
        return self._response_expected
    
    ## 監視時に期待されるレスポンスコード
    response_expected = property(get_response_expected, set_response_expected, None)
    
    # (instance field) _active_connections
    
    ## @return {int}
    def get_active_connections(self):
        return self._active_connections
    
    ## 現在の接続数
    active_connections = property(get_active_connections, None, None)
    
    # (instance field) _status
    
    ## @return {str}
    def get_status(self):
        return self._status
    
    ## 現在の状態
    status = property(get_status, None, None)
    
    ## @ignore
    # @param {any} obj=None
    def __init__(self, obj=None):
        if obj is None:
            obj = {}
        health = Util.get_by_path_any([obj], ["HealthCheck", "healthCheck", "health_check", "health"])
        self._ip_address = Util.get_by_path_any([obj], ["IPAddress", "ipAddress", "ip_address", "ip"])
        self._protocol = Util.get_by_path_any([health, obj], ["Protocol", "protocol"])
        self._path_to_check = Util.get_by_path_any([health, obj], ["Path", "path", "pathToCheck", "path_to_check"])
        port = Util.get_by_path_any([obj], ["Port", "port"])
        self._port = None
        if port is not None:
            self._port = int(port)
        if self._port == 0:
            self._port = None
        responseExpected = Util.get_by_path_any([health, obj], ["Status", "status", "responseExpected", "response_expected"])
        self._response_expected = None
        if responseExpected is not None:
            self._response_expected = int(responseExpected)
        if self._response_expected == 0:
            self._response_expected = None
    
    ## @return {any}
    def to_raw_settings(self):
        return {
            'IPAddress': self._ip_address,
            'Port': self._port,
            'HealthCheck': {
                'Protocol': self._protocol,
                'Path': self._path_to_check,
                'Status': self._response_expected
            }
        }
    
    ## @param {any} obj
    # @return {saklient.cloud.resources.lbserver.LbServer}
    def update_status(self, obj):
        connStr = (obj["ActiveConn"] if "ActiveConn" in obj else None)
        self._active_connections = 0
        if connStr is not None:
            self._active_connections = int(connStr)
        status = (obj["Status"] if "Status" in obj else None)
        self._status = None if status is None else status.lower()
        return self
    



# -*- coding:utf-8 -*-

from ...util import Util

# module saklient.cloud.resources.ftpinfo

class FtpInfo:
    ## FTPサーバのアカウント情報。
    
    # (instance field) _host_name
    
    ## @return {str}
    def get_host_name(self):
        return self._host_name
    
    ## ホスト名
    host_name = property(get_host_name, None, None)
    
    # (instance field) _user
    
    ## @return {str}
    def get_user(self):
        return self._user
    
    ## ユーザ名
    user = property(get_user, None, None)
    
    # (instance field) _password
    
    ## @return {str}
    def get_password(self):
        return self._password
    
    ## パスワード
    password = property(get_password, None, None)
    
    ## @ignore
    # @param {any} obj
    def __init__(self, obj):
        self._host_name = (obj["HostName"] if "HostName" in obj else None)
        self._user = (obj["User"] if "User" in obj else None)
        self._password = (obj["Password"] if "Password" in obj else None)
    



# -*- coding:utf-8 -*-

from ..client import Client
from .model import Model
from ..resources.resource import Resource
from ..resources.archive import Archive
from ..resources.loadbalancer import LoadBalancer
from ..resources.vpcrouter import VpcRouter
from ..enums.escope import EScope
from ...util import Util

# module saklient.cloud.models.model_archive

class Model_Archive(Model):
    ## アーカイブを検索・作成するための機能を備えたクラス。
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/archive"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Archive"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Archives"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Archive"
    
    ## @private
    # @param {any} obj
    # @param {bool} wrapped=False
    # @return {saklient.cloud.resources.resource.Resource}
    def _create_resource_impl(self, obj, wrapped=False):
        Util.validate_type(wrapped, "bool")
        return Archive(self._client, obj, wrapped)
    
    ## 次に取得するリストの開始オフセットを指定します。
    # 
    # @param {int} offset オフセット
    # @return {saklient.cloud.models.model_archive.Model_Archive} this
    def offset(self, offset):
        Util.validate_type(offset, "int")
        return self._offset(offset)
    
    ## 次に取得するリストの上限レコード数を指定します。
    # 
    # @param {int} count 上限レコード数
    # @return {saklient.cloud.models.model_archive.Model_Archive} this
    def limit(self, count):
        Util.validate_type(count, "int")
        return self._limit(count)
    
    ## Web APIのフィルタリング設定を直接指定します。
    # 
    # @param {str} key キー
    # @param {any} value 値
    # @param {bool} multiple=False valueに配列を与え、OR条件で完全一致検索する場合にtrueを指定します。通常、valueはスカラ値であいまい検索されます。
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def filter_by(self, key, value, multiple=False):
        Util.validate_type(key, "str")
        Util.validate_type(multiple, "bool")
        return self._filter_by(key, value, multiple)
    
    ## 次のリクエストのために設定されているステートをすべて破棄します。
    # 
    # @return {saklient.cloud.models.model_archive.Model_Archive} this
    def reset(self):
        return self._reset()
    
    ## 新規リソース作成用のオブジェクトを用意します。
    # 
    # 返り値のオブジェクトにパラメータを設定し、save() を呼ぶことで実際のリソースが作成されます。
    # 
    # @return {saklient.cloud.resources.archive.Archive} リソースオブジェクト
    def create(self):
        return self._create()
    
    ## 指定したIDを持つ唯一のリソースを取得します。
    # 
    # @param {str} id
    # @return {saklient.cloud.resources.archive.Archive} リソースオブジェクト
    def get_by_id(self, id):
        Util.validate_type(id, "str")
        return self._get_by_id(id)
    
    ## リソースの検索リクエストを実行し、結果をリストで取得します。
    # 
    # @return {saklient.cloud.resources.archive.Archive[]} リソースオブジェクトの配列
    def find(self):
        return self._find()
    
    ## 指定した文字列を名前に含むリソースに絞り込みます。
    # 
    # 大文字・小文字は区別されません。
    # 半角スペースで区切られた複数の文字列は、それらをすべて含むことが条件とみなされます。
    # 
    # @todo Implement test case
    # @param {str} name
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def with_name_like(self, name):
        Util.validate_type(name, "str")
        return self._with_name_like(name)
    
    ## 指定したタグを持つリソースに絞り込みます。
    # 
    # 複数のタグを指定する場合は withTags() を利用してください。
    # 
    # @todo Implement test case
    # @param {str} tag
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def with_tag(self, tag):
        Util.validate_type(tag, "str")
        return self._with_tag(tag)
    
    ## 指定したすべてのタグを持つリソースに絞り込みます。
    # 
    # @todo Implement test case
    # @param {str[]} tags
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def with_tags(self, tags):
        Util.validate_type(tags, "list")
        return self._with_tags(tags)
    
    ## 指定したDNFに合致するタグを持つリソースに絞り込みます。
    # 
    # @todo Implement test case
    # @param {str[][]} dnf
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def with_tag_dnf(self, dnf):
        Util.validate_type(dnf, "list")
        return self._with_tag_dnf(dnf)
    
    ## 名前でソートします。
    # 
    # @todo Implement test case
    # @param {bool} reverse=False
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def sort_by_name(self, reverse=False):
        Util.validate_type(reverse, "bool")
        return self._sort_by_name(reverse)
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    def __init__(self, client):
        super(Model_Archive, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
    
    ## 指定したサイズのアーカイブに絞り込みます。
    # 
    # @param {int} sizeGib
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def with_size_gib(self, sizeGib):
        Util.validate_type(sizeGib, "int")
        self._filter_by("SizeMB", [sizeGib * 1024])
        return self
    
    ## パブリックアーカイブに絞り込みます。
    # 
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def with_shared_scope(self):
        self._filter_by("Scope", [EScope.shared])
        return self
    
    ## プライベートアーカイブに絞り込みます。
    # 
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def with_user_scope(self):
        self._filter_by("Scope", [EScope.user])
        return self
    
    ## サイズでソートします。
    # 
    # @param {bool} reverse=False
    # @return {saklient.cloud.models.model_archive.Model_Archive}
    def sort_by_size(self, reverse=False):
        Util.validate_type(reverse, "bool")
        self._sort("SizeMB", reverse)
        return self
    



# -*- coding:utf-8 -*-

from ...util import Util

# module saklient.cloud.enums.ediskconnection

class EDiskConnection:
    ## ディスクの接続方式を表す列挙子。
    
    IDE = "ide"
    ide = "ide"
    
    VIRTIO = "virtio"
    virtio = "virtio"
    
    ## @ignore
    _MAP = {
        "IDE": 100,
        "VIRTIO": 300
    }
    
    ## @ignore
    @classmethod
    def compare(clazz, lhs, rhs):
        if not (isinstance(lhs, str) and isinstance(rhs, str)): return None
        lhs = lhs.upper()
        rhs = rhs.upper()
        if lhs not in clazz._MAP or rhs not in clazz._MAP: return None
        d = clazz._MAP[lhs] - clazz._MAP[rhs]
        return (0<d) - (d<0)
    
    



# -*- coding:utf-8 -*-


# module saklient.errors.saklientexception

class SaklientException(Exception):
    
    # (instance field) code
    
    ## @param {str} code=None
    # @param {str} message=""
    def __init__(self, code=None, message=""):
        super(SaklientException, self).__init__(message)
        self.code = code
    



# -*- coding:utf-8 -*-

from ..client import Client
from ..resources.resource import Resource
from .queryparams import QueryParams
from ...errors.saklientexception import SaklientException
from ...util import Util

# module saklient.cloud.models.model

class Model:
    ## @ignore
    
    # (instance field) _client
    
    ## @return {saklient.cloud.client.Client}
    def get_client(self):
        return self._client
    
    client = property(get_client, None, None)
    
    # (instance field) _query
    
    ## @return {saklient.cloud.models.queryparams.QueryParams}
    def get_query(self):
        return self._query
    
    query = property(get_query, None, None)
    
    # (instance field) _total
    
    ## @return {int}
    def get_total(self):
        return self._total
    
    total = property(get_total, None, None)
    
    # (instance field) _count
    
    ## @return {int}
    def get_count(self):
        return self._count
    
    count = property(get_count, None, None)
    
    ## @private
    # @return {str}
    def _api_path(self):
        return None
    
    ## @private
    # @return {str}
    def _root_key(self):
        return None
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return None
    
    ## @private
    # @return {str}
    def _class_name(self):
        return None
    
    ## @param {saklient.cloud.client.Client} client
    def __init__(self, client):
        Util.validate_type(client, "saklient.cloud.client.Client")
        self._client = client
        self._reset()
    
    ## 次に取得するリストの開始オフセットを指定します。
    # 
    # @private
    # @param {int} offset オフセット
    # @return {saklient.cloud.models.model.Model} this
    def _offset(self, offset):
        Util.validate_type(offset, "int")
        self._query.begin = offset
        return self
    
    ## 次に取得するリストの上限レコード数を指定します。
    # 
    # @private
    # @param {int} count 上限レコード数
    # @return {saklient.cloud.models.model.Model} this
    def _limit(self, count):
        Util.validate_type(count, "int")
        self._query.count = count
        return self
    
    ## 次に取得するリストのソートカラムを指定します。
    # 
    # @private
    # @param {str} column カラム名
    # @param {bool} reverse=False
    # @return {saklient.cloud.models.model.Model} this
    def _sort(self, column, reverse=False):
        Util.validate_type(column, "str")
        Util.validate_type(reverse, "bool")
        op = "-" if reverse else ""
        self._query.sort.append(op + column)
        return self
    
    ## Web APIのフィルタリング設定を直接指定します。
    # 
    # @private
    # @param {str} key キー
    # @param {any} value 値
    # @param {bool} multiple=False valueに配列を与え、OR条件で完全一致検索する場合にtrueを指定します。通常、valueはスカラ値であいまい検索されます。
    # @return {saklient.cloud.models.model.Model}
    def _filter_by(self, key, value, multiple=False):
        Util.validate_type(key, "str")
        Util.validate_type(multiple, "bool")
        filter = self._query.filter
        if multiple:
            if not ( key in filter if isinstance(filter, dict) else hasattr(filter, key)):
                filter[key] = []
            values = (filter[key] if key in filter else None)
            values.append(value)
        else:
            if ( key in filter if isinstance(filter, dict) else hasattr(filter, key)):
                raise SaklientException("filter_duplicated", "The same filter key can be specified only once (by calling the same method 'withFooBar')")
            filter[key] = value
        return self
    
    ## 次のリクエストのために設定されているステートをすべて破棄します。
    # 
    # @private
    # @return {saklient.cloud.models.model.Model} this
    def _reset(self):
        self._query = QueryParams()
        self._total = 0
        self._count = 0
        return self
    
    ## @private
    # @ignore
    # @param {any} obj
    # @param {bool} wrapped=False
    # @return {saklient.cloud.resources.resource.Resource}
    def _create_resource_impl(self, obj, wrapped=False):
        Util.validate_type(wrapped, "bool")
        return None
    
    ## 新規リソース作成用のオブジェクトを用意します。
    # 
    # 返り値のオブジェクトにパラメータを設定し、save() を呼ぶことで実際のリソースが作成されます。
    # 
    # @private
    # @return {saklient.cloud.resources.resource.Resource} リソースオブジェクト
    def _create(self):
        return self._create_resource_impl(None)
    
    ## 指定したIDを持つ唯一のリソースを取得します。
    # 
    # @private
    # @param {str} id
    # @return {saklient.cloud.resources.resource.Resource} リソースオブジェクト
    def _get_by_id(self, id):
        Util.validate_type(id, "str")
        query = self._query.build()
        self._reset()
        result = self._client.request("GET", self._api_path() + "/" + Util.url_encode(id), query)
        self._total = 1
        self._count = 1
        return self._create_resource_impl(result, True)
    
    ## リソースの検索リクエストを実行し、結果をリストで取得します。
    # 
    # @private
    # @return {saklient.cloud.resources.resource.Resource[]} リソースオブジェクトの配列
    def _find(self):
        query = self._query.build()
        self._reset()
        result = self._client.request("GET", self._api_path(), query)
        self._total = (result["Total"] if "Total" in result else None)
        self._count = (result["Count"] if "Count" in result else None)
        data = []
        records = (result[self._root_key_m()] if self._root_key_m() in result else None)
        for record in records:
            data.append(self._create_resource_impl(record))
        return Client.haxe2native(data, 1)
    
    ## リソースの検索リクエストを実行し、唯一のリソースを取得します。
    # 
    # @private
    # @return {saklient.cloud.resources.resource.Resource} リソースオブジェクト
    def _find_one(self):
        query = self._query.build()
        self._reset()
        result = self._client.request("GET", self._api_path(), query)
        self._total = (result["Total"] if "Total" in result else None)
        self._count = (result["Count"] if "Count" in result else None)
        if self._total == 0:
            return None
        records = (result[self._root_key_m()] if self._root_key_m() in result else None)
        return self._create_resource_impl(records[0])
    
    ## 指定した文字列を名前に含むリソースに絞り込みます。
    # 
    # 大文字・小文字は区別されません。
    # 半角スペースで区切られた複数の文字列は、それらをすべて含むことが条件とみなされます。
    # 
    # @private
    # @todo Implement test case
    # @param {str} name
    # @return {saklient.cloud.models.model.Model}
    def _with_name_like(self, name):
        Util.validate_type(name, "str")
        return self._filter_by("Name", name)
    
    ## 指定したタグを持つリソースに絞り込みます。
    # 
    # 複数のタグを指定する場合は withTags() を利用してください。
    # 
    # @private
    # @todo Implement test case
    # @param {str} tag
    # @return {saklient.cloud.models.model.Model}
    def _with_tag(self, tag):
        Util.validate_type(tag, "str")
        return self._filter_by("Tags.Name", [tag])
    
    ## 指定したすべてのタグを持つリソースに絞り込みます。
    # 
    # @private
    # @todo Implement test case
    # @param {str[]} tags
    # @return {saklient.cloud.models.model.Model}
    def _with_tags(self, tags):
        Util.validate_type(tags, "list")
        return self._filter_by("Tags.Name", [tags])
    
    ## 指定したDNFに合致するタグを持つリソースに絞り込みます。
    # 
    # @private
    # @todo Implement test case
    # @param {str[][]} dnf
    # @return {saklient.cloud.models.model.Model}
    def _with_tag_dnf(self, dnf):
        Util.validate_type(dnf, "list")
        return self._filter_by("Tags.Name", dnf)
    
    ## 名前でソートします。
    # 
    # @private
    # @todo Implement test case
    # @param {bool} reverse=False
    # @return {saklient.cloud.models.model.Model}
    def _sort_by_name(self, reverse=False):
        Util.validate_type(reverse, "bool")
        return self._sort("Name", reverse)
    



# -*- coding:utf-8 -*-

import datetime, re, types, time, urllib.parse
from .errors.saklientexception import SaklientException

# module saklient.util

class Util:
    
    ## @ignore
    @staticmethod
    def exists_path(obj, path):
        a_path = path.split(".")
        for seg in a_path:
            if obj is None: return False
            if not isinstance(obj, dict): return False
            if seg == "": continue
            if seg not in obj: return False
            obj = obj[seg]
        return True
    
    ## @ignore
    @staticmethod
    def get_by_path(obj, path):
        a_path = path.split(".")
        for seg in a_path:
            if obj is None: return None
            if not isinstance(obj, dict): return None
            if seg == "": continue
            if seg not in obj: return None
            obj = obj[seg]
        return obj
  
    ## @ignore
    @staticmethod
    def get_by_path_any(objects, pathes):
        for obj in objects:
            for path in pathes:
                ret = Util.get_by_path(obj, path)
                if ret is not None: return ret
        return None
    
    ## @ignore
    @staticmethod
    def set_by_path(obj, path, value):
        a_path = path.split(".")
        key = a_path.pop()
        for seg in a_path:
            if seg == "": continue
            if seg not in obj: obj[seg] = {}
            obj = obj[seg]
        obj[key] = value
    
    ## @ignore
    @staticmethod
    def create_class_instance(classPath, arguments):
        ret = None
        p = classPath.split(".")
        c = p.pop()
        m = __import__(classPath.lower(), fromlist=[c])
        ret = getattr(m, c)(*arguments)
        if ret is None:
            raise Exception.new("Could not create class instance of " + classPath)
        return ret
    
    ## @ignore
    @staticmethod
    def str2date(s):
        if s is None:
            return None
        return datetime.datetime.strptime(re.sub(r"([+-][0-9]{2}):([0-9]{2})$", "\\1\\2", s), "%Y-%m-%dT%H:%M:%S%z")
    
    ## @ignore
    @staticmethod
    def date2str(d):
        if d is None:
            return None
        return d.strftime("%Y-%m-%dT%H:%M:%S%z")
    
    ## @ignore
    @staticmethod
    def url_encode(s):
        return urllib.parse.quote_plus(s)
    
    ## @ignore
    @staticmethod
    def sleep(sec):
        time.sleep(sec)
    
    ## @ignore
    @staticmethod
    def validate_arg_count(actual, expected):
        if actual < expected: raise SaklientException('argument_count_mismatch', 'Argument count mismatch')
    
    ## @ignore
    @staticmethod
    def validate_type(value, type_name, force=False):
        is_ok = False
        if not force:
            if type_name=="any" or type_name=="void" or value is None: return
            clazz = type(value)
            if (type_name == "function"):
                is_ok = isinstance(value, types.FunctionType)
            elif (type_name in {"str", "float", "int", "bool", "dict", "list"}):
                is_ok = type(value).__name__ == type_name
            else:
                p = type_name.split(".")
                if 1 < len(p):
                    c = p.pop()
                    m = __import__(".".join(p), fromlist=[c])
                    is_ok = isinstance(value, getattr(m, c))
                else:
                    c = globals()["__builtins__"][type_name]
                    is_ok = isinstance(value, c)
            
        if not is_ok:
            raise SaklientException("argument_type_mismatch", "Argument type mismatch (expected "+type_name+", got "+str(type(value))+")")




# -*- coding:utf-8 -*-

from ...util import Util

# module saklient.cloud.enums.escope

class EScope:
    ## リソースの公開範囲を表す列挙子。
    
    USER = "user"
    user = "user"
    
    SHARED = "shared"
    shared = "shared"
    
    ## @ignore
    _MAP = {
        "USER": 100,
        "SHARED": 200
    }
    
    ## @ignore
    @classmethod
    def compare(clazz, lhs, rhs):
        if not (isinstance(lhs, str) and isinstance(rhs, str)): return None
        lhs = lhs.upper()
        rhs = rhs.upper()
        if lhs not in clazz._MAP or rhs not in clazz._MAP: return None
        d = clazz._MAP[lhs] - clazz._MAP[rhs]
        return (0<d) - (d<0)
    
    



# -*- coding:utf-8 -*-

from .httpexception import HttpException

# module saklient.errors.httprequesttimeoutexception

class HttpRequestTimeoutException(HttpException):
    ## HTTPエラー。Request Timeout.
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(HttpRequestTimeoutException, self).__init__(status, code, "HTTPエラー。Request Timeout." if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.httpconflictexception import HttpConflictException

# module saklient.cloud.errors.packetfilterapplyingexception

class PacketFilterApplyingException(HttpConflictException):
    ## 要求された操作を行えません。起動中のサーバに対して変更されたパケットフィルタを反映するタスクが既に実行中です。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(PacketFilterApplyingException, self).__init__(status, code, "要求された操作を行えません。起動中のサーバに対して変更されたパケットフィルタを反映するタスクが既に実行中です。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ..client import Client
from .model import Model
from ..resources.resource import Resource
from ..resources.server import Server
from ..resources.serverplan import ServerPlan
from ..resources.isoimage import IsoImage
from ..enums.eserverinstancestatus import EServerInstanceStatus
from ...util import Util

# module saklient.cloud.models.model_server

class Model_Server(Model):
    ## サーバを検索・作成するための機能を備えたクラス。
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/server"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Server"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Servers"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Server"
    
    ## @private
    # @param {any} obj
    # @param {bool} wrapped=False
    # @return {saklient.cloud.resources.resource.Resource}
    def _create_resource_impl(self, obj, wrapped=False):
        Util.validate_type(wrapped, "bool")
        return Server(self._client, obj, wrapped)
    
    ## 次に取得するリストの開始オフセットを指定します。
    # 
    # @param {int} offset オフセット
    # @return {saklient.cloud.models.model_server.Model_Server} this
    def offset(self, offset):
        Util.validate_type(offset, "int")
        return self._offset(offset)
    
    ## 次に取得するリストの上限レコード数を指定します。
    # 
    # @param {int} count 上限レコード数
    # @return {saklient.cloud.models.model_server.Model_Server} this
    def limit(self, count):
        Util.validate_type(count, "int")
        return self._limit(count)
    
    ## Web APIのフィルタリング設定を直接指定します。
    # 
    # @param {str} key キー
    # @param {any} value 値
    # @param {bool} multiple=False valueに配列を与え、OR条件で完全一致検索する場合にtrueを指定します。通常、valueはスカラ値であいまい検索されます。
    # @return {saklient.cloud.models.model_server.Model_Server}
    def filter_by(self, key, value, multiple=False):
        Util.validate_type(key, "str")
        Util.validate_type(multiple, "bool")
        return self._filter_by(key, value, multiple)
    
    ## 次のリクエストのために設定されているステートをすべて破棄します。
    # 
    # @return {saklient.cloud.models.model_server.Model_Server} this
    def reset(self):
        return self._reset()
    
    ## 新規リソース作成用のオブジェクトを用意します。
    # 
    # 返り値のオブジェクトにパラメータを設定し、save() を呼ぶことで実際のリソースが作成されます。
    # 
    # @return {saklient.cloud.resources.server.Server} リソースオブジェクト
    def create(self):
        return self._create()
    
    ## 指定したIDを持つ唯一のリソースを取得します。
    # 
    # @param {str} id
    # @return {saklient.cloud.resources.server.Server} リソースオブジェクト
    def get_by_id(self, id):
        Util.validate_type(id, "str")
        return self._get_by_id(id)
    
    ## リソースの検索リクエストを実行し、結果をリストで取得します。
    # 
    # @return {saklient.cloud.resources.server.Server[]} リソースオブジェクトの配列
    def find(self):
        return self._find()
    
    ## 指定した文字列を名前に含むリソースに絞り込みます。
    # 
    # 大文字・小文字は区別されません。
    # 半角スペースで区切られた複数の文字列は、それらをすべて含むことが条件とみなされます。
    # 
    # @todo Implement test case
    # @param {str} name
    # @return {saklient.cloud.models.model_server.Model_Server}
    def with_name_like(self, name):
        Util.validate_type(name, "str")
        return self._with_name_like(name)
    
    ## 指定したタグを持つリソースに絞り込みます。
    # 
    # 複数のタグを指定する場合は withTags() を利用してください。
    # 
    # @todo Implement test case
    # @param {str} tag
    # @return {saklient.cloud.models.model_server.Model_Server}
    def with_tag(self, tag):
        Util.validate_type(tag, "str")
        return self._with_tag(tag)
    
    ## 指定したすべてのタグを持つリソースに絞り込みます。
    # 
    # @todo Implement test case
    # @param {str[]} tags
    # @return {saklient.cloud.models.model_server.Model_Server}
    def with_tags(self, tags):
        Util.validate_type(tags, "list")
        return self._with_tags(tags)
    
    ## 指定したDNFに合致するタグを持つリソースに絞り込みます。
    # 
    # @todo Implement test case
    # @param {str[][]} dnf
    # @return {saklient.cloud.models.model_server.Model_Server}
    def with_tag_dnf(self, dnf):
        Util.validate_type(dnf, "list")
        return self._with_tag_dnf(dnf)
    
    ## 名前でソートします。
    # 
    # @todo Implement test case
    # @param {bool} reverse=False
    # @return {saklient.cloud.models.model_server.Model_Server}
    def sort_by_name(self, reverse=False):
        Util.validate_type(reverse, "bool")
        return self._sort_by_name(reverse)
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    def __init__(self, client):
        super(Model_Server, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
    
    ## 指定したプランのサーバに絞り込みます。
    # 
    # @param {saklient.cloud.resources.serverplan.ServerPlan} plan
    # @return {saklient.cloud.models.model_server.Model_Server}
    def with_plan(self, plan):
        Util.validate_type(plan, "saklient.cloud.resources.serverplan.ServerPlan")
        self._filter_by("ServerPlan.ID", [plan._id()])
        return self
    
    ## インスタンスが指定した状態にあるサーバに絞り込みます。
    # 
    # @param {str} status
    # @return {saklient.cloud.models.model_server.Model_Server}
    def with_status(self, status):
        Util.validate_type(status, "str")
        self._filter_by("Instance.Status", [status])
        return self
    
    ## インスタンスが起動中のサーバに絞り込みます。
    # 
    # @return {saklient.cloud.models.model_server.Model_Server}
    def with_status_up(self):
        return self.with_status(EServerInstanceStatus.up)
    
    ## インスタンスが停止中のサーバに絞り込みます。
    # 
    # @return {saklient.cloud.models.model_server.Model_Server}
    def with_status_down(self):
        return self.with_status(EServerInstanceStatus.down)
    
    ## 指定したISOイメージが挿入されているサーバに絞り込みます。
    # 
    # @param {saklient.cloud.resources.isoimage.IsoImage} iso
    # @return {saklient.cloud.models.model_server.Model_Server}
    def with_iso_image(self, iso):
        Util.validate_type(iso, "saklient.cloud.resources.isoimage.IsoImage")
        self._filter_by("Instance.CDROM.ID", [iso._id()])
        return self
    
    ## 仮想コア数でソートします。
    # 
    # @param {bool} reverse=False
    # @return {saklient.cloud.models.model_server.Model_Server}
    def sort_by_cpu(self, reverse=False):
        Util.validate_type(reverse, "bool")
        self._sort("ServerPlan.CPU", reverse)
        return self
    
    ## メモリ容量でソートします。
    # 
    # @param {bool} reverse=False
    # @return {saklient.cloud.models.model_server.Model_Server}
    def sort_by_memory(self, reverse=False):
        Util.validate_type(reverse, "bool")
        self._sort("ServerPlan.MemoryMB", reverse)
        return self
    



# -*- coding:utf-8 -*-

from ...util import Util
from ..client import Client
from .script import Script

# module saklient.cloud.resources.diskconfig

class DiskConfig:
    ## ディスク修正のパラメータ。
    
    # (instance field) _client
    
    ## @return {saklient.cloud.client.Client}
    def get_client(self):
        return self._client
    
    ## @ignore
    client = property(get_client, None, None)
    
    # (instance field) _disk_id
    
    ## @return {str}
    def get_disk_id(self):
        return self._disk_id
    
    ## @ignore
    disk_id = property(get_disk_id, None, None)
    
    # (instance field) _host_name
    
    ## @return {str}
    def get_host_name(self):
        return self._host_name
    
    ## @param {str} v
    # @return {str}
    def set_host_name(self, v):
        Util.validate_type(v, "str")
        self._host_name = v
        return v
    
    ## ホスト名
    host_name = property(get_host_name, set_host_name, None)
    
    # (instance field) _password
    
    ## @return {str}
    def get_password(self):
        return self._password
    
    ## @param {str} v
    # @return {str}
    def set_password(self, v):
        Util.validate_type(v, "str")
        self._password = v
        return v
    
    ## ログインパスワード
    password = property(get_password, set_password, None)
    
    # (instance field) _ssh_key
    
    ## @return {str}
    def get_ssh_key(self):
        return self._ssh_key
    
    ## @param {str} v
    # @return {str}
    def set_ssh_key(self, v):
        Util.validate_type(v, "str")
        self._ssh_key = v
        return v
    
    ## SSHキー
    ssh_key = property(get_ssh_key, set_ssh_key, None)
    
    # (instance field) _ip_address
    
    ## @return {str}
    def get_ip_address(self):
        return self._ip_address
    
    ## @param {str} v
    # @return {str}
    def set_ip_address(self, v):
        Util.validate_type(v, "str")
        self._ip_address = v
        return v
    
    ## IPアドレス
    ip_address = property(get_ip_address, set_ip_address, None)
    
    # (instance field) _default_route
    
    ## @return {str}
    def get_default_route(self):
        return self._default_route
    
    ## @param {str} v
    # @return {str}
    def set_default_route(self, v):
        Util.validate_type(v, "str")
        self._default_route = v
        return v
    
    ## デフォルトルート
    default_route = property(get_default_route, set_default_route, None)
    
    # (instance field) _network_mask_len
    
    ## @return {int}
    def get_network_mask_len(self):
        return self._network_mask_len
    
    ## @param {int} v
    # @return {int}
    def set_network_mask_len(self, v):
        Util.validate_type(v, "int")
        self._network_mask_len = v
        return v
    
    ## ネットワークマスク長
    network_mask_len = property(get_network_mask_len, set_network_mask_len, None)
    
    # (instance field) _scripts
    
    ## @return {saklient.cloud.resources.script.Script[]}
    def get_scripts(self):
        return self._scripts
    
    ## スタートアップスクリプト {@link Script} の配列（pushによりスクリプトを追加できます）
    scripts = property(get_scripts, None, None)
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {str} diskId
    def __init__(self, client, diskId):
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(diskId, "str")
        self._client = client
        self._disk_id = diskId
        self._host_name = None
        self._password = None
        self._ssh_key = None
        self._ip_address = None
        self._default_route = None
        self._network_mask_len = None
        self._scripts = []
    
    ## スタートアップスクリプトを追加します。
    # 
    # diskConfig.addScript(script) と diskConfig.scripts.push(script) の効果は同等です。
    # 
    # @param {saklient.cloud.resources.script.Script} script
    # @return {saklient.cloud.resources.diskconfig.DiskConfig} this
    def add_script(self, script):
        Util.validate_type(script, "saklient.cloud.resources.script.Script")
        self._scripts.append(script)
        return self
    
    ## 修正内容を実際のディスクに書き込みます。
    # 
    # @return {saklient.cloud.resources.diskconfig.DiskConfig} this
    def write(self):
        q = {}
        if self._host_name is not None:
            Util.set_by_path(q, "HostName", self._host_name)
        if self._password is not None:
            Util.set_by_path(q, "Password", self._password)
        if self._ssh_key is not None:
            Util.set_by_path(q, "SSHKey.PublicKey", self._ssh_key)
        if self._ip_address is not None:
            Util.set_by_path(q, "UserIPAddress", self._ip_address)
        if self._default_route is not None:
            Util.set_by_path(q, "UserSubnet.DefaultRoute", self._default_route)
        if self._network_mask_len is not None:
            Util.set_by_path(q, "UserSubnet.NetworkMaskLen", self._network_mask_len)
        if 0 < len(self._scripts):
            notes = []
            for script in self._scripts:
                notes.append({
                    'ID': script._id()
                })
            Util.set_by_path(q, "Notes", notes)
        path = "/disk/" + self._disk_id + "/config"
        self._client.request("PUT", path, q)
        return self
    



# -*- coding:utf-8 -*-

from ...errors.saklientexception import SaklientException
from ..client import Client
from .resource import Resource
from .icon import Icon
from .diskplan import DiskPlan
from .server import Server
from .diskconfig import DiskConfig
from ..enums.eavailability import EAvailability
from ..enums.ediskconnection import EDiskConnection
from ..enums.estorageclass import EStorageClass
from ...util import Util

# module saklient.cloud.resources.disk

class Disk(Resource):
    ## ディスクの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_name
    
    # (instance field) m_description
    
    # (instance field) m_tags
    
    # (instance field) m_icon
    
    # (instance field) m_size_mib
    
    # (instance field) m_service_class
    
    # (instance field) m_plan
    
    # (instance field) m_server
    
    # (instance field) m_availability
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/disk"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Disk"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Disks"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Disk"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## このローカルオブジェクトに現在設定されているリソース情報をAPIに送信し、新規作成または上書き保存します。
    # 
    # @return {saklient.cloud.resources.disk.Disk} this
    def save(self):
        return self._save()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {saklient.cloud.resources.disk.Disk} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(Disk, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    ## @return {bool}
    def get_is_available(self):
        return self.get_availability() == EAvailability.available
    
    ## ディスクが利用可能なときtrueを返します。
    is_available = property(get_is_available, None, None)
    
    ## @return {int}
    def get_size_gib(self):
        sizeMib = self.get_size_mib()
        return None if sizeMib is None else sizeMib >> 10
    
    ## @param {int} sizeGib
    # @return {int}
    def set_size_gib(self, sizeGib):
        Util.validate_type(sizeGib, "int")
        self.set_size_mib(None if sizeGib is None else sizeGib * 1024)
        return sizeGib
    
    ## サイズ[GiB]
    size_gib = property(get_size_gib, set_size_gib, None)
    
    # (instance field) _source
    
    ## @return {saklient.cloud.resources.resource.Resource}
    def get_source(self):
        return self._source
    
    ## @param {saklient.cloud.resources.resource.Resource} source
    # @return {saklient.cloud.resources.resource.Resource}
    def set_source(self, source):
        Util.validate_type(source, "saklient.cloud.resources.resource.Resource")
        self._source = source
        return source
    
    ## ディスクのコピー元
    source = property(get_source, set_source, None)
    
    ## @private
    # @param {any} r
    # @param {any} root
    # @return {void}
    def _on_after_api_deserialize(self, r, root):
        if r is not None:
            if ( "SourceDisk" in r if isinstance(r, dict) else hasattr(r, "SourceDisk")):
                s = (r["SourceDisk"] if "SourceDisk" in r else None)
                if s is not None:
                    id = (s["ID"] if "ID" in s else None)
                    if id is not None:
                        self._source = Disk(self._client, s)
            if ( "SourceArchive" in r if isinstance(r, dict) else hasattr(r, "SourceArchive")):
                s = (r["SourceArchive"] if "SourceArchive" in r else None)
                if s is not None:
                    id = (s["ID"] if "ID" in s else None)
                    if id is not None:
                        self._source = Resource.create_with("Archive", self._client, s)
    
    ## @private
    # @param {any} r
    # @param {bool} withClean
    # @return {void}
    def _on_after_api_serialize(self, r, withClean):
        Util.validate_type(withClean, "bool")
        if r is None:
            return
        if self._source is not None:
            if self._source._class_name() == "Disk":
                s = self._source.api_serialize(True) if withClean else {
                    'ID': self._source._id()
                }
                r["SourceDisk"] = s
            else:
                if self._source._class_name() == "Archive":
                    s = self._source.api_serialize(True) if withClean else {
                        'ID': self._source._id()
                    }
                    r["SourceArchive"] = s
                else:
                    self._source = None
                    Util.validate_type(self._source, "Disk or Archive", True)
    
    ## ディスクをサーバに取り付けます。
    # 
    # @param {saklient.cloud.resources.server.Server} server
    # @return {saklient.cloud.resources.disk.Disk} this
    def connect_to(self, server):
        Util.validate_type(server, "saklient.cloud.resources.server.Server")
        self._client.request("PUT", "/disk/" + self._id() + "/to/server/" + server._id())
        return self
    
    ## ディスクをサーバから取り外します。
    # 
    # @return {saklient.cloud.resources.disk.Disk} this
    def disconnect(self):
        self._client.request("DELETE", "/disk/" + self._id() + "/to/server")
        return self
    
    ## ディスク修正を行うためのオブジェクトを用意します。
    # 
    # 返り値のオブジェクトにパラメータを設定し、write() を呼ぶことで修正を行います。
    # 
    # @return {saklient.cloud.resources.diskconfig.DiskConfig}
    def create_config(self):
        return DiskConfig(self._client, self._id())
    
    ## コピー中のディスクが利用可能になるまで待機します。
    # 
    # @param {int} timeoutSec=3600
    # @return {bool} 成功時はtrue、タイムアウトやエラーによる失敗時はfalseを返します。
    def sleep_while_copying(self, timeoutSec=3600):
        Util.validate_type(timeoutSec, "int")
        step = 10
        while (0 < timeoutSec):
            self.reload()
            a = self.get_availability()
            if a == EAvailability.available:
                return True
            if a != EAvailability.migrating:
                timeoutSec = 0
            timeoutSec -= step
            if 0 < timeoutSec:
                Util.sleep(step)
        return False
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_name = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_name(self):
        return self.m_name
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_name(self, v):
        Util.validate_type(v, "str")
        self.m_name = v
        self.n_name = True
        return self.m_name
    
    ## 名前
    name = property(get_name, set_name, None)
    
    # (instance field) n_description = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_description(self):
        return self.m_description
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_description(self, v):
        Util.validate_type(v, "str")
        self.m_description = v
        self.n_description = True
        return self.m_description
    
    ## 説明
    description = property(get_description, set_description, None)
    
    # (instance field) n_tags = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str[]}
    def get_tags(self):
        self.n_tags = True
        return self.m_tags
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str[]} v
    # @return {str[]}
    def set_tags(self, v):
        Util.validate_type(v, "list")
        self.m_tags = v
        self.n_tags = True
        return self.m_tags
    
    ## タグ文字列の配列
    tags = property(get_tags, set_tags, None)
    
    # (instance field) n_icon = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.icon.Icon}
    def get_icon(self):
        return self.m_icon
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {saklient.cloud.resources.icon.Icon} v
    # @return {saklient.cloud.resources.icon.Icon}
    def set_icon(self, v):
        Util.validate_type(v, "saklient.cloud.resources.icon.Icon")
        self.m_icon = v
        self.n_icon = True
        return self.m_icon
    
    ## アイコン
    icon = property(get_icon, set_icon, None)
    
    # (instance field) n_size_mib = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_size_mib(self):
        return self.m_size_mib
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {int} v
    # @return {int}
    def set_size_mib(self, v):
        Util.validate_type(v, "int")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.disk.Disk#size_mib")
        self.m_size_mib = v
        self.n_size_mib = True
        return self.m_size_mib
    
    ## サイズ[MiB]
    size_mib = property(get_size_mib, set_size_mib, None)
    
    # (instance field) n_service_class = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_service_class(self):
        return self.m_service_class
    
    ## サービスクラス
    service_class = property(get_service_class, None, None)
    
    # (instance field) n_plan = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.diskplan.DiskPlan}
    def get_plan(self):
        return self.m_plan
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {saklient.cloud.resources.diskplan.DiskPlan} v
    # @return {saklient.cloud.resources.diskplan.DiskPlan}
    def set_plan(self, v):
        Util.validate_type(v, "saklient.cloud.resources.diskplan.DiskPlan")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.disk.Disk#plan")
        self.m_plan = v
        self.n_plan = True
        return self.m_plan
    
    ## プラン
    plan = property(get_plan, set_plan, None)
    
    # (instance field) n_server = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.server.Server}
    def get_server(self):
        return self.m_server
    
    ## 接続先のサーバ
    server = property(get_server, None, None)
    
    # (instance field) n_availability = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_availability(self):
        return self.m_availability
    
    ## 有効状態 {@link EAvailability}
    availability = property(get_availability, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "Name"):
            self.m_name = None if Util.get_by_path(r, "Name") is None else str(Util.get_by_path(r, "Name"))
        else:
            self.m_name = None
            self.is_incomplete = True
        self.n_name = False
        if Util.exists_path(r, "Description"):
            self.m_description = None if Util.get_by_path(r, "Description") is None else str(Util.get_by_path(r, "Description"))
        else:
            self.m_description = None
            self.is_incomplete = True
        self.n_description = False
        if Util.exists_path(r, "Tags"):
            if Util.get_by_path(r, "Tags") is None:
                self.m_tags = []
            else:
                self.m_tags = []
                for t in Util.get_by_path(r, "Tags"):
                    v1 = None
                    v1 = None if t is None else str(t)
                    self.m_tags.append(v1)
        else:
            self.m_tags = None
            self.is_incomplete = True
        self.n_tags = False
        if Util.exists_path(r, "Icon"):
            self.m_icon = None if Util.get_by_path(r, "Icon") is None else Icon(self._client, Util.get_by_path(r, "Icon"))
        else:
            self.m_icon = None
            self.is_incomplete = True
        self.n_icon = False
        if Util.exists_path(r, "SizeMB"):
            self.m_size_mib = None if Util.get_by_path(r, "SizeMB") is None else int(str(Util.get_by_path(r, "SizeMB")))
        else:
            self.m_size_mib = None
            self.is_incomplete = True
        self.n_size_mib = False
        if Util.exists_path(r, "ServiceClass"):
            self.m_service_class = None if Util.get_by_path(r, "ServiceClass") is None else str(Util.get_by_path(r, "ServiceClass"))
        else:
            self.m_service_class = None
            self.is_incomplete = True
        self.n_service_class = False
        if Util.exists_path(r, "Plan"):
            self.m_plan = None if Util.get_by_path(r, "Plan") is None else DiskPlan(self._client, Util.get_by_path(r, "Plan"))
        else:
            self.m_plan = None
            self.is_incomplete = True
        self.n_plan = False
        if Util.exists_path(r, "Server"):
            self.m_server = None if Util.get_by_path(r, "Server") is None else Server(self._client, Util.get_by_path(r, "Server"))
        else:
            self.m_server = None
            self.is_incomplete = True
        self.n_server = False
        if Util.exists_path(r, "Availability"):
            self.m_availability = None if Util.get_by_path(r, "Availability") is None else str(Util.get_by_path(r, "Availability"))
        else:
            self.m_availability = None
            self.is_incomplete = True
        self.n_availability = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        missing = []
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_name:
            Util.set_by_path(ret, "Name", self.m_name)
        else:
            if self.is_new:
                missing.append("name")
        if withClean or self.n_description:
            Util.set_by_path(ret, "Description", self.m_description)
        if withClean or self.n_tags:
            Util.set_by_path(ret, "Tags", [])
            for r1 in self.m_tags:
                v = None
                v = r1
                (ret["Tags"] if "Tags" in ret else None).append(v)
        if withClean or self.n_icon:
            Util.set_by_path(ret, "Icon", (None if self.m_icon is None else self.m_icon.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_icon is None else self.m_icon.api_serialize_id()))
        if withClean or self.n_size_mib:
            Util.set_by_path(ret, "SizeMB", self.m_size_mib)
        if withClean or self.n_service_class:
            Util.set_by_path(ret, "ServiceClass", self.m_service_class)
        if withClean or self.n_plan:
            Util.set_by_path(ret, "Plan", (None if self.m_plan is None else self.m_plan.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_plan is None else self.m_plan.api_serialize_id()))
        else:
            if self.is_new:
                missing.append("plan")
        if withClean or self.n_server:
            Util.set_by_path(ret, "Server", (None if self.m_server is None else self.m_server.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_server is None else self.m_server.api_serialize_id()))
        if withClean or self.n_availability:
            Util.set_by_path(ret, "Availability", self.m_availability)
        if len(missing) > 0:
            raise SaklientException("required_field", "Required fields must be set before the Disk creation: " + ", ".join(missing))
        return ret
    



# -*- coding:utf-8 -*-

from ..client import Client
from .resource import Resource
from .icon import Icon
from .ftpinfo import FtpInfo
from ..enums.escope import EScope
from ...errors.saklientexception import SaklientException
from ...util import Util

# module saklient.cloud.resources.isoimage

class IsoImage(Resource):
    ## ISOイメージの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_scope
    
    # (instance field) m_name
    
    # (instance field) m_description
    
    # (instance field) m_tags
    
    # (instance field) m_icon
    
    # (instance field) m_display_order
    
    # (instance field) m_size_mib
    
    # (instance field) m_service_class
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/cdrom"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "CDROM"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "CDROMs"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "IsoImage"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## このローカルオブジェクトに現在設定されているリソース情報をAPIに送信し、新規作成または上書き保存します。
    # 
    # @return {saklient.cloud.resources.isoimage.IsoImage} this
    def save(self):
        return self._save()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {saklient.cloud.resources.isoimage.IsoImage} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(IsoImage, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    ## @private
    # @param {any} r
    # @param {any} root
    # @return {void}
    def _on_after_api_deserialize(self, r, root):
        if root is None:
            return
        if ( "FTPServer" in root if isinstance(root, dict) else hasattr(root, "FTPServer")):
            ftp = (root["FTPServer"] if "FTPServer" in root else None)
            if ftp is not None:
                self._ftp_info = FtpInfo(ftp)
    
    ## @return {int}
    def get_size_gib(self):
        sizeMib = self.get_size_mib()
        return None if sizeMib is None else sizeMib >> 10
    
    ## @param {int} sizeGib
    # @return {int}
    def set_size_gib(self, sizeGib):
        Util.validate_type(sizeGib, "int")
        self.set_size_mib(None if sizeGib is None else sizeGib * 1024)
        return sizeGib
    
    ## サイズ[GiB]
    size_gib = property(get_size_gib, set_size_gib, None)
    
    # (instance field) _ftp_info
    
    ## @return {saklient.cloud.resources.ftpinfo.FtpInfo}
    def get_ftp_info(self):
        return self._ftp_info
    
    ## FTP情報
    ftp_info = property(get_ftp_info, None, None)
    
    ## FTPSを開始し、イメージファイルをアップロード・ダウンロードできる状態にします。
    # 
    # アカウント情報は、ftpInfo プロパティから取得することができます。
    # 
    # @param {bool} reset=False 既にFTPSが開始されているとき、trueを指定してこのメソッドを呼ぶことでパスワードを再設定します。
    # @return {saklient.cloud.resources.isoimage.IsoImage} this
    def open_ftp(self, reset=False):
        Util.validate_type(reset, "bool")
        path = self._api_path() + "/" + Util.url_encode(self._id()) + "/ftp"
        q = {}
        Util.set_by_path(q, "ChangePassword", reset)
        result = self._client.request("PUT", path, q)
        self._on_after_api_deserialize(None, result)
        return self
    
    ## FTPSを終了し、ISOイメージを利用可能な状態にします。
    # 
    # @return {saklient.cloud.resources.isoimage.IsoImage} this
    def close_ftp(self):
        path = self._api_path() + "/" + Util.url_encode(self._id()) + "/ftp"
        self._client.request("DELETE", path)
        self._ftp_info = None
        return self
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_scope = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_scope(self):
        return self.m_scope
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_scope(self, v):
        Util.validate_type(v, "str")
        self.m_scope = v
        self.n_scope = True
        return self.m_scope
    
    ## スコープ {@link EScope}
    scope = property(get_scope, set_scope, None)
    
    # (instance field) n_name = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_name(self):
        return self.m_name
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_name(self, v):
        Util.validate_type(v, "str")
        self.m_name = v
        self.n_name = True
        return self.m_name
    
    ## 名前
    name = property(get_name, set_name, None)
    
    # (instance field) n_description = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_description(self):
        return self.m_description
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_description(self, v):
        Util.validate_type(v, "str")
        self.m_description = v
        self.n_description = True
        return self.m_description
    
    ## 説明
    description = property(get_description, set_description, None)
    
    # (instance field) n_tags = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str[]}
    def get_tags(self):
        self.n_tags = True
        return self.m_tags
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str[]} v
    # @return {str[]}
    def set_tags(self, v):
        Util.validate_type(v, "list")
        self.m_tags = v
        self.n_tags = True
        return self.m_tags
    
    ## タグ文字列の配列
    tags = property(get_tags, set_tags, None)
    
    # (instance field) n_icon = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.icon.Icon}
    def get_icon(self):
        return self.m_icon
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {saklient.cloud.resources.icon.Icon} v
    # @return {saklient.cloud.resources.icon.Icon}
    def set_icon(self, v):
        Util.validate_type(v, "saklient.cloud.resources.icon.Icon")
        self.m_icon = v
        self.n_icon = True
        return self.m_icon
    
    ## アイコン
    icon = property(get_icon, set_icon, None)
    
    # (instance field) n_display_order = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_display_order(self):
        return self.m_display_order
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {int} v
    # @return {int}
    def set_display_order(self, v):
        Util.validate_type(v, "int")
        self.m_display_order = v
        self.n_display_order = True
        return self.m_display_order
    
    ## 表示順序
    display_order = property(get_display_order, set_display_order, None)
    
    # (instance field) n_size_mib = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_size_mib(self):
        return self.m_size_mib
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {int} v
    # @return {int}
    def set_size_mib(self, v):
        Util.validate_type(v, "int")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.isoimage.IsoImage#size_mib")
        self.m_size_mib = v
        self.n_size_mib = True
        return self.m_size_mib
    
    ## サイズ[MiB]
    size_mib = property(get_size_mib, set_size_mib, None)
    
    # (instance field) n_service_class = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_service_class(self):
        return self.m_service_class
    
    ## サービスクラス
    service_class = property(get_service_class, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "Scope"):
            self.m_scope = None if Util.get_by_path(r, "Scope") is None else str(Util.get_by_path(r, "Scope"))
        else:
            self.m_scope = None
            self.is_incomplete = True
        self.n_scope = False
        if Util.exists_path(r, "Name"):
            self.m_name = None if Util.get_by_path(r, "Name") is None else str(Util.get_by_path(r, "Name"))
        else:
            self.m_name = None
            self.is_incomplete = True
        self.n_name = False
        if Util.exists_path(r, "Description"):
            self.m_description = None if Util.get_by_path(r, "Description") is None else str(Util.get_by_path(r, "Description"))
        else:
            self.m_description = None
            self.is_incomplete = True
        self.n_description = False
        if Util.exists_path(r, "Tags"):
            if Util.get_by_path(r, "Tags") is None:
                self.m_tags = []
            else:
                self.m_tags = []
                for t in Util.get_by_path(r, "Tags"):
                    v1 = None
                    v1 = None if t is None else str(t)
                    self.m_tags.append(v1)
        else:
            self.m_tags = None
            self.is_incomplete = True
        self.n_tags = False
        if Util.exists_path(r, "Icon"):
            self.m_icon = None if Util.get_by_path(r, "Icon") is None else Icon(self._client, Util.get_by_path(r, "Icon"))
        else:
            self.m_icon = None
            self.is_incomplete = True
        self.n_icon = False
        if Util.exists_path(r, "DisplayOrder"):
            self.m_display_order = None if Util.get_by_path(r, "DisplayOrder") is None else int(str(Util.get_by_path(r, "DisplayOrder")))
        else:
            self.m_display_order = None
            self.is_incomplete = True
        self.n_display_order = False
        if Util.exists_path(r, "SizeMB"):
            self.m_size_mib = None if Util.get_by_path(r, "SizeMB") is None else int(str(Util.get_by_path(r, "SizeMB")))
        else:
            self.m_size_mib = None
            self.is_incomplete = True
        self.n_size_mib = False
        if Util.exists_path(r, "ServiceClass"):
            self.m_service_class = None if Util.get_by_path(r, "ServiceClass") is None else str(Util.get_by_path(r, "ServiceClass"))
        else:
            self.m_service_class = None
            self.is_incomplete = True
        self.n_service_class = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        missing = []
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_scope:
            Util.set_by_path(ret, "Scope", self.m_scope)
        if withClean or self.n_name:
            Util.set_by_path(ret, "Name", self.m_name)
        else:
            if self.is_new:
                missing.append("name")
        if withClean or self.n_description:
            Util.set_by_path(ret, "Description", self.m_description)
        if withClean or self.n_tags:
            Util.set_by_path(ret, "Tags", [])
            for r1 in self.m_tags:
                v = None
                v = r1
                (ret["Tags"] if "Tags" in ret else None).append(v)
        if withClean or self.n_icon:
            Util.set_by_path(ret, "Icon", (None if self.m_icon is None else self.m_icon.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_icon is None else self.m_icon.api_serialize_id()))
        if withClean or self.n_display_order:
            Util.set_by_path(ret, "DisplayOrder", self.m_display_order)
        if withClean or self.n_size_mib:
            Util.set_by_path(ret, "SizeMB", self.m_size_mib)
        else:
            if self.is_new:
                missing.append("size_mib")
        if withClean or self.n_service_class:
            Util.set_by_path(ret, "ServiceClass", self.m_service_class)
        if len(missing) > 0:
            raise SaklientException("required_field", "Required fields must be set before the IsoImage creation: " + ", ".join(missing))
        return ret
    



# -*- coding:utf-8 -*-

from ...errors.httpconflictexception import HttpConflictException

# module saklient.cloud.errors.duplicateaccountcodeexception

class DuplicateAccountCodeException(HttpConflictException):
    ## 要求された操作を行えません。同一アカウント名で複数のアカウントを作成することはできません。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(DuplicateAccountCodeException, self).__init__(status, code, "要求された操作を行えません。同一アカウント名で複数のアカウントを作成することはできません。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.httpserviceunavailableexception import HttpServiceUnavailableException

# module saklient.cloud.errors.servercouldnotstopexception

class ServerCouldNotStopException(HttpServiceUnavailableException):
    ## サービスが利用できません。サーバを停止できません。再度お試しください。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(ServerCouldNotStopException, self).__init__(status, code, "サービスが利用できません。サーバを停止できません。再度お試しください。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.saklientexception import SaklientException
from ..client import Client
from .resource import Resource
from .icon import Icon
from .router import Router
from .ipv4net import Ipv4Net
from .ipv6net import Ipv6Net
from .bridge import Bridge
from ...util import Util

# module saklient.cloud.resources.swytch

class Swytch(Resource):
    ## スイッチの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_name
    
    # (instance field) m_description
    
    # (instance field) m_tags
    
    # (instance field) m_icon
    
    # (instance field) m_user_default_route
    
    # (instance field) m_user_mask_len
    
    # (instance field) m_router
    
    # (instance field) m_bridge
    
    # (instance field) m_ipv4_nets
    
    # (instance field) m_ipv6_nets
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/switch"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Switch"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Switches"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Swytch"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## このローカルオブジェクトに現在設定されているリソース情報をAPIに送信し、新規作成または上書き保存します。
    # 
    # @return {saklient.cloud.resources.swytch.Swytch} this
    def save(self):
        return self._save()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {saklient.cloud.resources.swytch.Swytch} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(Swytch, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    ## このルータ＋スイッチでIPv6アドレスを有効にします。
    # 
    # @return {saklient.cloud.resources.ipv6net.Ipv6Net} 有効化されたIPv6ネットワーク
    def add_ipv6_net(self):
        ret = self.get_router().add_ipv6_net()
        self.reload()
        return ret
    
    ## このルータ＋スイッチでIPv6アドレスを無効にします。
    # 
    # @return {saklient.cloud.resources.swytch.Swytch} this
    def remove_ipv6_net(self):
        nets = self.get_ipv6_nets()
        self.get_router().remove_ipv6_net(nets[0])
        self.reload()
        return self
    
    ## このルータ＋スイッチにスタティックルートを追加します。
    # 
    # @param {int} maskLen
    # @param {str} nextHop
    # @return {saklient.cloud.resources.ipv4net.Ipv4Net} 追加されたIPv4ネットワーク
    def add_static_route(self, maskLen, nextHop):
        Util.validate_type(maskLen, "int")
        Util.validate_type(nextHop, "str")
        ret = self.get_router().add_static_route(maskLen, nextHop)
        self.reload()
        return ret
    
    ## このルータ＋スイッチからスタティックルートを削除します。
    # 
    # @param {saklient.cloud.resources.ipv4net.Ipv4Net} ipv4Net
    # @return {saklient.cloud.resources.swytch.Swytch} this
    def remove_static_route(self, ipv4Net):
        Util.validate_type(ipv4Net, "saklient.cloud.resources.ipv4net.Ipv4Net")
        self.get_router().remove_static_route(ipv4Net)
        self.reload()
        return self
    
    ## このルータ＋スイッチの帯域プランを変更します。
    # 
    # @param {int} bandWidthMbps 帯域幅（api.product.router.find() から取得できる {@link RouterPlan} の bandWidthMbps を指定）。
    # @return {saklient.cloud.resources.swytch.Swytch} this
    def change_plan(self, bandWidthMbps):
        Util.validate_type(bandWidthMbps, "int")
        self.get_router().change_plan(bandWidthMbps)
        self.reload()
        return self
    
    ## このルータ＋スイッチをブリッジに接続します。
    # 
    # @param swytch 接続先のブリッジ。
    # @param {saklient.cloud.resources.bridge.Bridge} bridge
    # @return {saklient.cloud.resources.swytch.Swytch} this
    def connect_to_bridge(self, bridge):
        Util.validate_type(bridge, "saklient.cloud.resources.bridge.Bridge")
        result = self._client.request("PUT", self._api_path() + "/" + self._id() + "/to/bridge/" + bridge._id())
        self.reload()
        return self
    
    ## このルータ＋スイッチをブリッジから切断します。
    # 
    # @return {saklient.cloud.resources.swytch.Swytch} this
    def disconnect_from_bridge(self):
        result = self._client.request("DELETE", self._api_path() + "/" + self._id() + "/to/bridge")
        self.reload()
        return self
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_name = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_name(self):
        return self.m_name
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_name(self, v):
        Util.validate_type(v, "str")
        self.m_name = v
        self.n_name = True
        return self.m_name
    
    ## 名前
    name = property(get_name, set_name, None)
    
    # (instance field) n_description = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_description(self):
        return self.m_description
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_description(self, v):
        Util.validate_type(v, "str")
        self.m_description = v
        self.n_description = True
        return self.m_description
    
    ## 説明
    description = property(get_description, set_description, None)
    
    # (instance field) n_tags = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str[]}
    def get_tags(self):
        self.n_tags = True
        return self.m_tags
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str[]} v
    # @return {str[]}
    def set_tags(self, v):
        Util.validate_type(v, "list")
        self.m_tags = v
        self.n_tags = True
        return self.m_tags
    
    ## タグ文字列の配列
    tags = property(get_tags, set_tags, None)
    
    # (instance field) n_icon = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.icon.Icon}
    def get_icon(self):
        return self.m_icon
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {saklient.cloud.resources.icon.Icon} v
    # @return {saklient.cloud.resources.icon.Icon}
    def set_icon(self, v):
        Util.validate_type(v, "saklient.cloud.resources.icon.Icon")
        self.m_icon = v
        self.n_icon = True
        return self.m_icon
    
    ## アイコン
    icon = property(get_icon, set_icon, None)
    
    # (instance field) n_user_default_route = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_user_default_route(self):
        return self.m_user_default_route
    
    ## ユーザ設定IPv4ネットワークのゲートウェイ
    user_default_route = property(get_user_default_route, None, None)
    
    # (instance field) n_user_mask_len = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_user_mask_len(self):
        return self.m_user_mask_len
    
    ## ユーザ設定IPv4ネットワークのマスク長
    user_mask_len = property(get_user_mask_len, None, None)
    
    # (instance field) n_router = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.router.Router}
    def get_router(self):
        return self.m_router
    
    ## 接続されているルータ
    router = property(get_router, None, None)
    
    # (instance field) n_bridge = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.bridge.Bridge}
    def get_bridge(self):
        return self.m_bridge
    
    ## 接続されているブリッジ
    bridge = property(get_bridge, None, None)
    
    # (instance field) n_ipv4_nets = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.ipv4net.Ipv4Net[]}
    def get_ipv4_nets(self):
        return self.m_ipv4_nets
    
    ## IPv4ネットワーク（ルータによる自動割当） {@link Ipv4Net} の配列
    ipv4_nets = property(get_ipv4_nets, None, None)
    
    # (instance field) n_ipv6_nets = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.ipv6net.Ipv6Net[]}
    def get_ipv6_nets(self):
        return self.m_ipv6_nets
    
    ## IPv6ネットワーク（ルータによる自動割当） {@link Ipv6Net} の配列
    ipv6_nets = property(get_ipv6_nets, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "Name"):
            self.m_name = None if Util.get_by_path(r, "Name") is None else str(Util.get_by_path(r, "Name"))
        else:
            self.m_name = None
            self.is_incomplete = True
        self.n_name = False
        if Util.exists_path(r, "Description"):
            self.m_description = None if Util.get_by_path(r, "Description") is None else str(Util.get_by_path(r, "Description"))
        else:
            self.m_description = None
            self.is_incomplete = True
        self.n_description = False
        if Util.exists_path(r, "Tags"):
            if Util.get_by_path(r, "Tags") is None:
                self.m_tags = []
            else:
                self.m_tags = []
                for t in Util.get_by_path(r, "Tags"):
                    v1 = None
                    v1 = None if t is None else str(t)
                    self.m_tags.append(v1)
        else:
            self.m_tags = None
            self.is_incomplete = True
        self.n_tags = False
        if Util.exists_path(r, "Icon"):
            self.m_icon = None if Util.get_by_path(r, "Icon") is None else Icon(self._client, Util.get_by_path(r, "Icon"))
        else:
            self.m_icon = None
            self.is_incomplete = True
        self.n_icon = False
        if Util.exists_path(r, "UserSubnet.DefaultRoute"):
            self.m_user_default_route = None if Util.get_by_path(r, "UserSubnet.DefaultRoute") is None else str(Util.get_by_path(r, "UserSubnet.DefaultRoute"))
        else:
            self.m_user_default_route = None
            self.is_incomplete = True
        self.n_user_default_route = False
        if Util.exists_path(r, "UserSubnet.NetworkMaskLen"):
            self.m_user_mask_len = None if Util.get_by_path(r, "UserSubnet.NetworkMaskLen") is None else int(str(Util.get_by_path(r, "UserSubnet.NetworkMaskLen")))
        else:
            self.m_user_mask_len = None
            self.is_incomplete = True
        self.n_user_mask_len = False
        if Util.exists_path(r, "Internet"):
            self.m_router = None if Util.get_by_path(r, "Internet") is None else Router(self._client, Util.get_by_path(r, "Internet"))
        else:
            self.m_router = None
            self.is_incomplete = True
        self.n_router = False
        if Util.exists_path(r, "Bridge"):
            self.m_bridge = None if Util.get_by_path(r, "Bridge") is None else Bridge(self._client, Util.get_by_path(r, "Bridge"))
        else:
            self.m_bridge = None
            self.is_incomplete = True
        self.n_bridge = False
        if Util.exists_path(r, "Subnets"):
            if Util.get_by_path(r, "Subnets") is None:
                self.m_ipv4_nets = []
            else:
                self.m_ipv4_nets = []
                for t in Util.get_by_path(r, "Subnets"):
                    v2 = None
                    v2 = None if t is None else Ipv4Net(self._client, t)
                    self.m_ipv4_nets.append(v2)
        else:
            self.m_ipv4_nets = None
            self.is_incomplete = True
        self.n_ipv4_nets = False
        if Util.exists_path(r, "IPv6Nets"):
            if Util.get_by_path(r, "IPv6Nets") is None:
                self.m_ipv6_nets = []
            else:
                self.m_ipv6_nets = []
                for t in Util.get_by_path(r, "IPv6Nets"):
                    v3 = None
                    v3 = None if t is None else Ipv6Net(self._client, t)
                    self.m_ipv6_nets.append(v3)
        else:
            self.m_ipv6_nets = None
            self.is_incomplete = True
        self.n_ipv6_nets = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        missing = []
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_name:
            Util.set_by_path(ret, "Name", self.m_name)
        else:
            if self.is_new:
                missing.append("name")
        if withClean or self.n_description:
            Util.set_by_path(ret, "Description", self.m_description)
        if withClean or self.n_tags:
            Util.set_by_path(ret, "Tags", [])
            for r1 in self.m_tags:
                v = None
                v = r1
                (ret["Tags"] if "Tags" in ret else None).append(v)
        if withClean or self.n_icon:
            Util.set_by_path(ret, "Icon", (None if self.m_icon is None else self.m_icon.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_icon is None else self.m_icon.api_serialize_id()))
        if withClean or self.n_user_default_route:
            Util.set_by_path(ret, "UserSubnet.DefaultRoute", self.m_user_default_route)
        if withClean or self.n_user_mask_len:
            Util.set_by_path(ret, "UserSubnet.NetworkMaskLen", self.m_user_mask_len)
        if withClean or self.n_router:
            Util.set_by_path(ret, "Internet", (None if self.m_router is None else self.m_router.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_router is None else self.m_router.api_serialize_id()))
        if withClean or self.n_bridge:
            Util.set_by_path(ret, "Bridge", (None if self.m_bridge is None else self.m_bridge.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_bridge is None else self.m_bridge.api_serialize_id()))
        if withClean or self.n_ipv4_nets:
            Util.set_by_path(ret, "Subnets", [])
            for r2 in self.m_ipv4_nets:
                v = None
                v = (None if r2 is None else r2.api_serialize(withClean)) if withClean else ({
                    'ID': "0"
                } if r2 is None else r2.api_serialize_id())
                (ret["Subnets"] if "Subnets" in ret else None).append(v)
        if withClean or self.n_ipv6_nets:
            Util.set_by_path(ret, "IPv6Nets", [])
            for r3 in self.m_ipv6_nets:
                v = None
                v = (None if r3 is None else r3.api_serialize(withClean)) if withClean else ({
                    'ID': "0"
                } if r3 is None else r3.api_serialize_id())
                (ret["IPv6Nets"] if "IPv6Nets" in ret else None).append(v)
        if len(missing) > 0:
            raise SaklientException("required_field", "Required fields must be set before the Swytch creation: " + ", ".join(missing))
        return ret
    



# -*- coding:utf-8 -*-

from ...errors.saklientexception import SaklientException
from ..client import Client
from .appliance import Appliance
from ...util import Util

# module saklient.cloud.resources.vpcrouter

class VpcRouter(Appliance):
    ## ロードバランサの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(VpcRouter, self).__init__(client, obj, wrapped)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
    



# -*- coding:utf-8 -*-

from ..util import Util
from .client import Client
from .product import Product
from .facility import Facility
from .models.model_icon import Model_Icon
from .models.model_server import Model_Server
from .models.model_disk import Model_Disk
from .models.model_appliance import Model_Appliance
from .models.model_archive import Model_Archive
from .models.model_isoimage import Model_IsoImage
from .models.model_iface import Model_Iface
from .models.model_swytch import Model_Swytch
from .models.model_router import Model_Router
from .models.model_bridge import Model_Bridge
from .models.model_ipv6net import Model_Ipv6Net
from .models.model_script import Model_Script
from .models.model_license import Model_License

# module saklient.cloud.api

class API:
    ## さくらのクラウドAPIクライアントを利用する際、最初にアクセスすべきルートとなるクラス。
    # 
    # @see API.authorize
    
    # (instance field) _client
    
    ## @return {saklient.cloud.client.Client}
    def get_client(self):
        return self._client
    
    ## @ignore
    client = property(get_client, None, None)
    
    # (instance field) _product
    
    ## @return {saklient.cloud.product.Product}
    def get_product(self):
        return self._product
    
    ## 商品情報にアクセスするためのモデルを集めたオブジェクト。
    product = property(get_product, None, None)
    
    # (instance field) _facility
    
    ## @return {saklient.cloud.facility.Facility}
    def get_facility(self):
        return self._facility
    
    ## 設備情報にアクセスするためのモデルを集めたオブジェクト。
    facility = property(get_facility, None, None)
    
    # (instance field) _icon
    
    ## @return {saklient.cloud.models.model_icon.Model_Icon}
    def get_icon(self):
        return self._icon
    
    ## アイコンにアクセスするためのモデル。
    icon = property(get_icon, None, None)
    
    # (instance field) _server
    
    ## @return {saklient.cloud.models.model_server.Model_Server}
    def get_server(self):
        return self._server
    
    ## サーバにアクセスするためのモデル。
    server = property(get_server, None, None)
    
    # (instance field) _disk
    
    ## @return {saklient.cloud.models.model_disk.Model_Disk}
    def get_disk(self):
        return self._disk
    
    ## ディスクにアクセスするためのモデル。
    disk = property(get_disk, None, None)
    
    # (instance field) _appliance
    
    ## @return {saklient.cloud.models.model_appliance.Model_Appliance}
    def get_appliance(self):
        return self._appliance
    
    ## アプライアンスにアクセスするためのモデル。
    appliance = property(get_appliance, None, None)
    
    # (instance field) _archive
    
    ## @return {saklient.cloud.models.model_archive.Model_Archive}
    def get_archive(self):
        return self._archive
    
    ## アーカイブにアクセスするためのモデル。
    archive = property(get_archive, None, None)
    
    # (instance field) _iso_image
    
    ## @return {saklient.cloud.models.model_isoimage.Model_IsoImage}
    def get_iso_image(self):
        return self._iso_image
    
    ## ISOイメージにアクセスするためのモデル。
    iso_image = property(get_iso_image, None, None)
    
    # (instance field) _iface
    
    ## @return {saklient.cloud.models.model_iface.Model_Iface}
    def get_iface(self):
        return self._iface
    
    ## インタフェースにアクセスするためのモデル。
    iface = property(get_iface, None, None)
    
    # (instance field) _swytch
    
    ## @return {saklient.cloud.models.model_swytch.Model_Swytch}
    def get_swytch(self):
        return self._swytch
    
    ## スイッチにアクセスするためのモデル。
    swytch = property(get_swytch, None, None)
    
    # (instance field) _router
    
    ## @return {saklient.cloud.models.model_router.Model_Router}
    def get_router(self):
        return self._router
    
    ## ルータにアクセスするためのモデル。
    router = property(get_router, None, None)
    
    # (instance field) _bridge
    
    ## @return {saklient.cloud.models.model_bridge.Model_Bridge}
    def get_bridge(self):
        return self._bridge
    
    ## ブリッジにアクセスするためのモデル。
    bridge = property(get_bridge, None, None)
    
    # (instance field) _ipv6_net
    
    ## @return {saklient.cloud.models.model_ipv6net.Model_Ipv6Net}
    def get_ipv6_net(self):
        return self._ipv6_net
    
    ## IPv6ネットワークにアクセスするためのモデル。
    ipv6_net = property(get_ipv6_net, None, None)
    
    # (instance field) _script
    
    ## @return {saklient.cloud.models.model_script.Model_Script}
    def get_script(self):
        return self._script
    
    ## スクリプトにアクセスするためのモデル。
    script = property(get_script, None, None)
    
    # (instance field) _license
    
    ## @return {saklient.cloud.models.model_license.Model_License}
    def get_license(self):
        return self._license
    
    ## ライセンスにアクセスするためのモデル。
    license = property(get_license, None, None)
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    def __init__(self, client):
        Util.validate_type(client, "saklient.cloud.client.Client")
        self._client = client
        self._product = Product(client)
        self._facility = Facility(client)
        self._icon = Model_Icon(client)
        self._server = Model_Server(client)
        self._disk = Model_Disk(client)
        self._appliance = Model_Appliance(client)
        self._archive = Model_Archive(client)
        self._iso_image = Model_IsoImage(client)
        self._iface = Model_Iface(client)
        self._swytch = Model_Swytch(client)
        self._router = Model_Router(client)
        self._bridge = Model_Bridge(client)
        self._ipv6_net = Model_Ipv6Net(client)
        self._script = Model_Script(client)
        self._license = Model_License(client)
    
    ## 指定した認証情報を用いてアクセスを行うAPIクライアントを作成します。
    # 
    # 必要な認証情報は、コントロールパネル右上にあるアカウントのプルダウンから
    # 「設定」を選択し、「APIキー」のページにて作成できます。
    # 
    # @static
    # @param {str} token ACCESS TOKEN
    # @param {str} secret ACCESS TOKEN SECRET
    # @param {str} zone=None ゾーン名
    # @return {saklient.cloud.api.API} APIクライアント
    @staticmethod
    def authorize(token, secret, zone=None):
        Util.validate_type(token, "str")
        Util.validate_type(secret, "str")
        Util.validate_type(zone, "str")
        c = Client(token, secret)
        ret = API(c)
        return ret.in_zone(zone) if zone is not None else ret
    
    ## 認証情報を引き継ぎ、指定したゾーンへのアクセスを行うAPIクライアントを作成します。
    # 
    # @param {str} name ゾーン名
    # @return {saklient.cloud.api.API} APIクライアント
    def in_zone(self, name):
        Util.validate_type(name, "str")
        ret = API(self._client.clone_instance())
        suffix = ""
        if name == "is1x" or name == "is1y":
            suffix = "-test"
        ret._client.set_api_root("https://secure.sakura.ad.jp/cloud" + suffix + "/")
        ret._client.set_api_root_suffix("zone/" + name)
        return ret
    



# -*- coding:utf-8 -*-

from .httpexception import HttpException

# module saklient.errors.httpconflictexception

class HttpConflictException(HttpException):
    ## 要求された操作を行えません。現在の対象の状態では、この操作を受け付けできません。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(HttpConflictException, self).__init__(status, code, "要求された操作を行えません。現在の対象の状態では、この操作を受け付けできません。" if message is None or message == "" else message)
    



# -*- coding: utf-8 -*-

import urllib.request, urllib.parse
from urllib.error import URLError, HTTPError
import json, re, base64
from ..errors.exceptionfactory import ExceptionFactory

class Client:
    
    def __init__(self, token, secret):
        self.config = {
            'api_root': 'https://secure.sakura.ad.jp/cloud/',
            'api_root_suffix': None
        }
        self.set_access_key(token, secret)
    
    @staticmethod
    def native2haxe(r, depth):
        return r
    
    @staticmethod
    def haxe2native(r, depth):
        return r

    def clone_instance(self):
      instance = self.__class__(self.config['token'], self.config['secret'])
      instance.set_api_root(self.config['api_root'])
      instance.set_api_root_suffix(self.config['api_root_suffix'])
      return instance

    def set_api_root(self, url):
        self.config['api_root'] = url

    def set_api_root_suffix(self, suffix):
        self.config['api_root_suffix'] = suffix

    def set_access_key(self, token, secret):
        self.config['token']  = token
        self.config['secret'] = secret
        auth_bytes = (token+':'+secret).encode('utf-8')
        self.config['authorization'] = 'Basic ' + base64.b64encode(auth_bytes).decode("utf-8")
    
    def request(self, method, path, params={}):
        method = method.upper()
        if path[0] != '/':
            path = '/' + path
        params_json = json.dumps(params).encode(encoding='ascii')
        if method == 'GET':
            if params_json is not None:
                path += '?' + urllib.parse.quote(params_json)
            params_json = None
        if path[0:4] != 'http':
            url_root = self.config['api_root']
            if self.config['api_root_suffix'] is not None:
                if re.compile('is1[v-z]').match(self.config['api_root_suffix']):
                    url_root = re.compile('/cloud/$').sub('/cloud-test/', url_root)
                url_root += self.config['api_root_suffix'];
                url_root = re.compile('/?$').sub('/', url_root)
                if url_root[-1:] != '/':
                    url_root += '/'
            path = url_root + 'api/cloud/1.1' + path
        
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Authorization': self.config['authorization'],
            'User-Agent': 'saklient.python ver-0.0.2.5 rev-a581b28f8ddd2cd75cb32f710d33d3d650bba044',
            'X-Requested-With': 'XMLHttpRequest',
            'X-Sakura-No-Authenticate-Header': '1',
            'X-Sakura-HTTP-Method': method,
            'X-Sakura-Request-Format': 'json',
            'X-Sakura-Response-Format': 'json',
            'X-Sakura-Error-Level': 'warning'
        }
        
        res = ''
        try:
            req = urllib.request.Request(path, params_json, headers)
            with urllib.request.urlopen(req) as page:
                for line in page.readlines():
                    res += line.decode('utf-8')
        except HTTPError as ex:
            res = ex.read().decode('utf8', 'ignore')
            ret = None
            try:
                ret = json.loads(res)
            except Exception as ex2:
                pass
            if not isinstance(ret, dict):
                ret = {"error_code":None, "error_msg":None}
            raise ExceptionFactory.create(ex.code, ret["error_code"], ret["error_msg"]);
        
        return json.loads(res)
    



# -*- coding:utf-8 -*-

from ..client import Client
from .model import Model
from ..resources.resource import Resource
from ..resources.diskplan import DiskPlan
from ...util import Util

# module saklient.cloud.models.model_diskplan

class Model_DiskPlan(Model):
    ## ディスクプランを検索するための機能を備えたクラス。
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/product/disk"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "DiskPlan"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "DiskPlans"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "DiskPlan"
    
    ## @private
    # @param {any} obj
    # @param {bool} wrapped=False
    # @return {saklient.cloud.resources.resource.Resource}
    def _create_resource_impl(self, obj, wrapped=False):
        Util.validate_type(wrapped, "bool")
        return DiskPlan(self._client, obj, wrapped)
    
    ## 次に取得するリストの開始オフセットを指定します。
    # 
    # @param {int} offset オフセット
    # @return {saklient.cloud.models.model_diskplan.Model_DiskPlan} this
    def offset(self, offset):
        Util.validate_type(offset, "int")
        return self._offset(offset)
    
    ## 次に取得するリストの上限レコード数を指定します。
    # 
    # @param {int} count 上限レコード数
    # @return {saklient.cloud.models.model_diskplan.Model_DiskPlan} this
    def limit(self, count):
        Util.validate_type(count, "int")
        return self._limit(count)
    
    ## Web APIのフィルタリング設定を直接指定します。
    # 
    # @param {str} key キー
    # @param {any} value 値
    # @param {bool} multiple=False valueに配列を与え、OR条件で完全一致検索する場合にtrueを指定します。通常、valueはスカラ値であいまい検索されます。
    # @return {saklient.cloud.models.model_diskplan.Model_DiskPlan}
    def filter_by(self, key, value, multiple=False):
        Util.validate_type(key, "str")
        Util.validate_type(multiple, "bool")
        return self._filter_by(key, value, multiple)
    
    ## 次のリクエストのために設定されているステートをすべて破棄します。
    # 
    # @return {saklient.cloud.models.model_diskplan.Model_DiskPlan} this
    def reset(self):
        return self._reset()
    
    ## 指定したIDを持つ唯一のリソースを取得します。
    # 
    # @param {str} id
    # @return {saklient.cloud.resources.diskplan.DiskPlan} リソースオブジェクト
    def get_by_id(self, id):
        Util.validate_type(id, "str")
        return self._get_by_id(id)
    
    ## リソースの検索リクエストを実行し、結果をリストで取得します。
    # 
    # @return {saklient.cloud.resources.diskplan.DiskPlan[]} リソースオブジェクトの配列
    def find(self):
        return self._find()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    def __init__(self, client):
        super(Model_DiskPlan, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        self._hdd = None
        self._ssd = None
    
    # (instance field) _hdd
    
    ## @return {saklient.cloud.resources.diskplan.DiskPlan}
    def get_hdd(self):
        if self._hdd is None:
            self._hdd = self.get_by_id("2")
        return self._hdd
    
    ## 標準プラン
    hdd = property(get_hdd, None, None)
    
    # (instance field) _ssd
    
    ## @return {saklient.cloud.resources.diskplan.DiskPlan}
    def get_ssd(self):
        if self._ssd is None:
            self._ssd = self.get_by_id("4")
        return self._ssd
    
    ## SSDプラン
    ssd = property(get_ssd, None, None)
    



# -*- coding:utf-8 -*-

from ...errors.httpinternalservererrorexception import HttpInternalServerErrorException

# module saklient.cloud.errors.unknownexception

class UnknownException(HttpInternalServerErrorException):
    ## 予期しないエラーが発生しました。このエラーが繰り返し発生する場合は、サポートサイトやメンテナンス情報をご確認ください。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(UnknownException, self).__init__(status, code, "予期しないエラーが発生しました。このエラーが繰り返し発生する場合は、サポートサイトやメンテナンス情報をご確認ください。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.httpnotfoundexception import HttpNotFoundException

# module saklient.cloud.errors.replicanotfoundexception

class ReplicaNotFoundException(HttpNotFoundException):
    ## 対象が見つかりません。このストレージには指定リソースの複製が存在しません。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(ReplicaNotFoundException, self).__init__(status, code, "対象が見つかりません。このストレージには指定リソースの複製が存在しません。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.httpforbiddenexception import HttpForbiddenException

# module saklient.cloud.errors.accesssakuraexception

class AccessSakuraException(HttpForbiddenException):
    ## 要求された操作は許可されていません。さくらインターネットの会員メニューより認証後に実行してください。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(AccessSakuraException, self).__init__(status, code, "要求された操作は許可されていません。さくらインターネットの会員メニューより認証後に実行してください。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.httpgatewaytimeoutexception import HttpGatewayTimeoutException

# module saklient.cloud.errors.apiproxytimeoutnongetexception

class ApiProxyTimeoutNonGetException(HttpGatewayTimeoutException):
    ## APIプロクシが応答しません。要求は実行された可能性があります。しばらく時間をおいてからご確認ください。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(ApiProxyTimeoutNonGetException, self).__init__(status, code, "APIプロクシが応答しません。要求は実行された可能性があります。しばらく時間をおいてからご確認ください。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.httpserviceunavailableexception import HttpServiceUnavailableException

# module saklient.cloud.errors.diskstockrunoutexception

class DiskStockRunOutException(HttpServiceUnavailableException):
    ## サービスが利用できません。作成済みディスクを確保できませんでした。サーバが混雑している可能性があります。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(DiskStockRunOutException, self).__init__(status, code, "サービスが利用できません。作成済みディスクを確保できませんでした。サーバが混雑している可能性があります。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...util import Util

# module saklient.cloud.enums.escriptclass

class EScriptClass:
    ## スクリプトのクラスを表す列挙子。
    
    SHELL = "shell"
    shell = "shell"
    
    ANSIBLE = "ansible"
    ansible = "ansible"
    
    ## @ignore
    _MAP = {
        "SHELL": 200,
        "ANSIBLE": 300
    }
    
    ## @ignore
    @classmethod
    def compare(clazz, lhs, rhs):
        if not (isinstance(lhs, str) and isinstance(rhs, str)): return None
        lhs = lhs.upper()
        rhs = rhs.upper()
        if lhs not in clazz._MAP or rhs not in clazz._MAP: return None
        d = clazz._MAP[lhs] - clazz._MAP[rhs]
        return (0<d) - (d<0)
    
    



# -*- coding:utf-8 -*-

from ...errors.httpconflictexception import HttpConflictException

# module saklient.cloud.errors.deletediskb4templateexception

class DeleteDiskB4TemplateException(HttpConflictException):
    ## 要求された操作を行えません。このテンプレートから作成したすべてのディスクを削除した後に実行してください。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(DeleteDiskB4TemplateException, self).__init__(status, code, "要求された操作を行えません。このテンプレートから作成したすべてのディスクを削除した後に実行してください。" if message is None or message == "" else message)
    
